﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.BasicComponents.Containers
{
    /// <summary>Serves as covariant (read-only) interface for a dictionary, where the key is a <see cref="IdentifierString"/>.
    /// </summary>
    /// <typeparam name="TValue">The type of the values in the dictionary.</typeparam>
    /// <remarks>In .NET 4.0 out parameters are not allowed for covariant generic types, thus <c>TryGetValue</c> methods are implemented
    /// as some extension methods. Moreover out parameters are allowed for interfaces only.</remarks>
    public interface IIdentifierStringDictionary<out TValue> : IEnumerable<TValue>
    {
        /// <summary>Gets the number of key/value pairs contained in the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/>.
        /// </summary>
        /// <value>The number of key/value pairs.</value>
        int Count
        {
            get;
        }

        /// <summary>Gets the (non-normalized) names of the objects in the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/> instance in its <see cref="System.String"/> representation.
        /// </summary>
        /// <value>The names of the items in its <see cref="System.String"/> representation.</value>
        IEnumerable<string> Names
        {
            get;
        }

        /// <summary>Gets the value associated with the specified key.
        /// </summary>
        /// <param name="key">The key of the value to get.</param>
        /// <value>The value associated with the specified key. If the specified key is not found, a get operation
        /// throws a <see cref="KeyNotFoundException"/>, and a set operation creates a new element with the specified key.</value>
        TValue this[string key]
        {
            get;
        }

        /// <summary>Gets the value associated with the specified key.
        /// </summary>
        /// <param name="key">The key of the value to get.</param>
        /// <value>The value associated with the specified key. If the specified key is not found, a get operation
        /// throws a <see cref="KeyNotFoundException"/>, and a set operation creates a new element with the specified key.</value>
        TValue this[IdentifierString key]
        {
            get;
        }

        /// <summary>Gets the names of the objects in the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/> instance in its <see cref="IdentifierString"/> representation.
        /// </summary>
        /// <returns>The names of the items in its <see cref="IdentifierString"/> representation.</returns>
        IEnumerable<IdentifierString> GetNamesAsIdentifierStrings();

        /// <summary>Determines whether the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/> contains the specified key.
        /// </summary>
        /// <param name="key">The key to locate in the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/>.</param>
        /// <returns><c>true</c> if the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/> contains an element with 
        /// the	specified key; otherwise, <c>false</c>.
        /// </returns>
        bool ContainsKey(string key);

        /// <summary>Determines whether the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/> contains the specified key.
        /// </summary>
        /// <param name="key">The key to locate in the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/>.</param>
        /// <returns><c>true</c> if the <see cref="IIdentifierStringDictionary&lt;TValue&gt;"/> contains an element with 
        /// the	specified key; otherwise, <c>false</c>.
        /// </returns>
        bool ContainsKey(IdentifierString key);
    }
}