/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.BasicComponents.LogFileFactory
{
    /// <summary>Represents an interface for collecting logfile messages which are not inserted
    /// in some <see cref="ILogFile"/> at that time but which can be inserted later.
    /// </summary>
    /// <remarks>This interface is used for wrapper classes which are used as input for some class which contains 
    /// some <see cref="ILogFile"/>. At the time of creating the wrapper class there is no <see cref="ILogFile"/> 
    /// instance available, i.e. it is not clear at this time for which class the wrapper is used as input. 
    /// The collected logfile messages are inserted into the logfile at the time of using the wrapper class as input.</remarks>
    public interface ICollectableLogFileMessages
    {
        /// <summary>Gets or sets a value indicating whether the current instances stores logfile messages
        /// for later use.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if the current instances store logfile messages; otherwise, <c>false</c>.
        /// </value>
        /// <remarks>Use this flag to switch off logfile message creation.</remarks>
        bool StoreLogFileMessages
        {
            get;
            set;
        }

        /// <summary>Removes stored logfile messages.
        /// </summary>
        void ClearStoredLogFileMessages();

        /// <summary>Writes the messages into logfile if <see cref="ICollectableLogFileMessages.StoreLogFileMessages"/>
        /// is <c>true</c>.
        /// </summary>
        /// <param name="logFile">The logfile to write the message strings.</param>
        /// <remarks>Each message which was stored in the current instance will be inserted into <paramref name="logFile"/>.</remarks>
        void WriteMessagesIntoLogFile(ILogFile logFile);
    }
}