/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.BasicComponents.LogFileFactory
{
    /// <summary>Serves as interface for a logfile.
    /// </summary>
    /// <remarks>This interface does not contain any output methods. Casting to the concrete implementation or
    /// some link to the <see cref="ILogFileGenerator"/> is needed for the output.</remarks>
    public interface ILogFile
    {
        #region properties

        /// <summary>Gets the type of the logfile output.
        /// </summary>
        /// <value>The type of the logfile output.</value>
        LogFileOutputType LogFileOutputType
        {
            get;
        }

        /// <summary>Gets the object type ID.
        /// </summary>
        /// <value>The object type ID.</value>
        Guid ObjectTypeID
        {
            get;
        }
        #endregion

        #region methods

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        void AddMessage(InfoLogFileMessageType messageType, string message);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        void AddMessage(InfoLogFileMessageType messageType);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        void AddMessage(WarningLogFileMessageType messageType, string message);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        void AddMessage(WarningLogFileMessageType messageType);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        void AddMessage(ErrorLogFileMessageType messageType, string message);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        void AddMessage(ErrorLogFileMessageType messageType);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        void AddMessage(FatalErrorLogFileMessageType messageType, string message);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        void AddMessage(FatalErrorLogFileMessageType messageType);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        /// <param name="message">The message.</param>
        void AddMessage(ILogFileMessageType messageType, string message);

        /// <summary>Adds a specific message into the logfile.
        /// </summary>
        /// <param name="messageType">The type of the message.</param>
        void AddMessage(ILogFileMessageType messageType);
        #endregion
    }
}