﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2013 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Windows.Forms;
using System.Collections.Generic;

using Dodoni.BasicComponents.Utilities;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.BasicComponents.ConfigurationFiles
{
    public partial class ConfigurationFileExample : Form
    {
        #region private members

        private ConfigurationFile m_ConfigurationFile;

        private SortedSet<string> m_PropertyCollectionNames = new SortedSet<string>();
        #endregion

        public ConfigurationFileExample()
        {
            InitializeComponent();
        }

        private void bLoad_Click(object sender, EventArgs e)
        {
            m_ConfigurationFile = ConfigurationFile.Create(editFileName.Text, "settings");
            m_PropertyCollectionNames.Clear();

            UpdateInfoOutputPropertyCollectionNames();
        }

        private void bAddProperty_Click(object sender, EventArgs e)
        {
            var propertyCollection = m_ConfigurationFile.GetPropertyCollection(editPropertyCollectionName.Text);
            propertyCollection.SetValue(editPropertyName.Text, editPropertyValue.Text);

            m_PropertyCollectionNames.Add(editPropertyCollectionName.Text);
            UpdateInfoOutputPropertyCollectionNames();
        }

        private void bWriteConfigFile_Click(object sender, EventArgs e)
        {
            if (m_ConfigurationFile != null)
            {
                m_ConfigurationFile.Save();
            }
        }

        private void cBoxInfoOutputPropertyNames_SelectedIndexChanged(object sender, EventArgs e)
        {
            var propertyCollection = m_ConfigurationFile.GetPropertyCollection((string)cBoxInfoOutputPropertyCollectionName.SelectedItem);

            InfoOutput infoOutput = new InfoOutput();
            propertyCollection.FillInfoOutput(infoOutput);

            var infoPackage = infoOutput.GetGeneralPackage();

            editInfoOutputPropertyValue.Text = infoPackage.GeneralProperties[(string)cBoxInfoOutputPropertyNames.SelectedItem].Value.ToString();
        }

        private void UpdateInfoOutputPropertyCollectionNames()
        {
            cBoxInfoOutputPropertyCollectionName.Items.Clear();

            foreach (var propertyCollectionName in m_PropertyCollectionNames)
            {
                cBoxInfoOutputPropertyCollectionName.Items.Add(propertyCollectionName);
            }
        }

        private void cBoxInfoOutputPropertyCollectionName_SelectedIndexChanged(object sender, EventArgs e)
        {
            var propertyCollection = m_ConfigurationFile.GetPropertyCollection((string)cBoxInfoOutputPropertyCollectionName.SelectedItem);

            InfoOutput infoOutput = new InfoOutput();
            propertyCollection.FillInfoOutput(infoOutput);

            var infoPackage = infoOutput.GetGeneralPackage();

            cBoxInfoOutputPropertyNames.Items.Clear();

            foreach (var property in infoPackage.GeneralProperties)
            {
                cBoxInfoOutputPropertyNames.Items.Add(property.Name.String);
            }
        }

        private void bAddExampleTable_Click(object sender, EventArgs e)
        {
            var table = m_ConfigurationFile.GetTable("Buildings", "House", "Size", "Rooms", "City");

            table.Items.Append("51", "2 1/2", "Berlin");
            table.Items.Append("120", "5", "London");
        }

        private void bShowExampleTable_Click(object sender, EventArgs e)
        {
            var table = m_ConfigurationFile.GetTable("Buildings", "House", "Size", "Rooms", "City");

            StringBuilder strBuilder = new StringBuilder();
            foreach (var row in table.Items)
            {
                strBuilder.AppendFormat("Size: {0}; Rooms: {1}; City: {2}", row[0], row[1], row[2]);
                strBuilder.AppendLine();
            }
            MessageBox.Show(strBuilder.ToString());
        }
    }
}