﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using NUnit.Framework;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.BasicComponents.Containers
{
    /// <summary>Serves as unit test class for <see cref="IdentifierStringDictionary&lt;TValue&gt;"/>.
    /// </summary>
    [TestFixture]
    public class IdentifierStringDictionaryTests
    {
        #region private members

        /// <summary>The object to test.
        /// </summary>
        private IdentifierStringDictionary<object> m_IdentifierStringDictionary;
        #endregion

        /// <summary>Setup the unit tests.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            m_IdentifierStringDictionary = IdentifierStringDictionary.Create<object>("Name of first item", 42.12, "Name of second item", true);
        }

        /// <summary>A test function that first adds an additional item and calls <see cref="IdentifierStringDictionaryBase&lt;TValue&gt;.TryGetValue(string, out TValue)"/>.
        /// </summary>
        [Test]
        public void AddItem_TryGetValue_TestCase()
        {
            m_IdentifierStringDictionary.Add("TestItem", 1234);

            object testOutput;
            bool state = m_IdentifierStringDictionary.TryGetValue("TestItem", out testOutput);

            Assert.That(state, NUnit.Framework.Is.EqualTo(true));
            Assert.That(testOutput, NUnit.Framework.Is.EqualTo(1234));
        }

        /// <summary>A test function for <see cref="IdentifierStringDictionaryBase&lt;TValue&gt;.TryGetValue(string, out TValue)"/> that queries the first item that has been added.
        /// </summary>
        [Test]
        public void TryGetValue_FirstItem_True()
        {
            object testOutput;
            bool state = m_IdentifierStringDictionary.TryGetValue("Name of first item", out testOutput);

            Assert.That(state, NUnit.Framework.Is.EqualTo(true));
        }

        /// <summary>A test function for <see cref="IdentifierStringDictionary&lt;TValue&gt;.Remove(string)"/>.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void Remove_FirstItem_InvalidOperationException()
        {
            // by default it is allowed to add items, but removing a item throws an exception:
            m_IdentifierStringDictionary.Remove("Name of first item");
        }
    }
}