﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using NUnit.Framework.Constraints;

namespace Dodoni.BasicComponents.Containers
{
    /// <summary>The constraint class that provides comparision methods for <see cref="InfoOutputProperty"/> objects with respect to the NUnit framework.
    /// </summary>
    internal class InfoOutputPropertyIsEqualToConstraint : Constraint
    {
        #region private members

        /// <summary>The expected <see cref="InfoOutputProperty"/> object.
        /// </summary>
        private InfoOutputProperty m_ExpectedInfoOutputProperty;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="InfoOutputPropertyIsEqualToConstraint"/> class.
        /// </summary>
        /// <param name="expectedInfoOutputProperty">The expected <see cref="InfoOutputProperty"/> object.</param>
        internal InfoOutputPropertyIsEqualToConstraint(InfoOutputProperty expectedInfoOutputProperty)
        {
            m_ExpectedInfoOutputProperty = expectedInfoOutputProperty;
        }
        #endregion

        #region public methods

        /// <summary>Test whether the constraint is satisfied by a given value.
        /// </summary>
        /// <param name="actual">The value to be tested.</param>
        /// <returns>True for success, false for failure.</returns>
        public override bool Matches(object actual)
        {
            base.actual = actual;

            if ((actual is InfoOutputProperty) == false)
            {
                return false;
            }
            if ((m_ExpectedInfoOutputProperty == null) && (actual == null))
            {
                return true;
            }
            InfoOutputProperty actualInfoOutputProperty = (InfoOutputProperty)actual;

            return ((m_ExpectedInfoOutputProperty.Name.Equals(actualInfoOutputProperty.Name)) && (m_ExpectedInfoOutputProperty.Value.Equals(actualInfoOutputProperty.Value)));
        }

        /// <summary>Write the constraint description to a MessageWriter.
        /// </summary>
        /// <param name="writer">The writer on which the description is displayed.</param>
        public override void WriteDescriptionTo(MessageWriter writer)
        {
            writer.WriteExpectedValue(m_ExpectedInfoOutputProperty);
        }

        /// <summary>Write the failure message to the MessageWriter provided
        /// as an argument. The default implementation simply passes
        /// the constraint and the actual value to the writer, which
        /// then displays the constraint description and the value.
        /// Constraints that need to provide additional details,
        /// such as where the error occured can override this.
        /// </summary>
        /// <param name="writer">The MessageWriter on which to display the message.</param>
        public override void WriteMessageTo(MessageWriter writer)
        {
            writer.DisplayDifferences(m_ExpectedInfoOutputProperty, actual);
        }
        #endregion
    }
}