﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using NUnit.Framework;
using Dodoni.BasicComponents.Utilities;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>A unit test class for <see cref="EnumExtensions"/>.
    /// </summary>
    [TestFixture]
    public class EnumExtensionsTests
    {
        /// <summary>Prepares the unit tests.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            EnumString.FlagsEnumSeparatorChar = ',';
        }

        /// <summary>A test function that compares the result of <see cref="EnumExtensions.ToFormatString(Enum, EnumStringRepresentationUsage)"/> with respect to <see cref="EnumStringRepresentationUsage.ToStringMethod"/>.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The <see cref="System.String"/> representation of <paramref name="value"/>.</returns>
        [TestCase(Weekdays.Sunday, Result = "Sunday")]
        [TestCase(Weekdays.Monday, Result = "Monday")]
        [TestCase(Weekdays.Tuesday, Result = "Tuesday")]
        [TestCase(StringAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(StringAttributeWeekdays.Monday, Result = "Monday")]
        [TestCase(StringAttributeWeekdays.Tuesday, Result = "Tuesday")]
        [TestCase(LanguageAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(LanguageAttributeWeekdays.Monday, Result = "Monday")]
        [TestCase(LanguageAttributeWeekdays.Tuesday, Result = "Tuesday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Monday, Result = "Monday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Tuesday, Result = "Tuesday")]
        [TestCase(PartialAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(PartialAttributeWeekdays.Monday, Result = "Monday")]
        [TestCase(PartialAttributeWeekdays.Tuesday, Result = "Tuesday")]
        [TestCase(FileAccess.Read, Result = "Read")]
        [TestCase(FileAccess.Write, Result = "Write")]
        [TestCase(FileAccess.Delete, Result = "Delete")]
        [TestCase(FileAccess.Read | FileAccess.Write | FileAccess.Delete, Result = "ReadWrite, Delete")]
        [TestCase(FileAccess.ReadWrite, Result = "ReadWrite")]
        [TestCase(StringAttributeFileAccess.Read, Result = "Read")]
        [TestCase(StringAttributeFileAccess.Write, Result = "Write")]
        [TestCase(StringAttributeFileAccess.Delete, Result = "Delete")]
        [TestCase(StringAttributeFileAccess.Read | StringAttributeFileAccess.Write | StringAttributeFileAccess.Delete, Result = "ReadWrite, Delete")]
        [TestCase(StringAndLanguageAttributeFileAccess.ReadWrite, Result = "ReadWrite")]
        [TestCase(StringAndLanguageAttributeFileAccess.Read, Result = "Read")]
        [TestCase(StringAndLanguageAttributeFileAccess.Write, Result = "Write")]
        [TestCase(StringAndLanguageAttributeFileAccess.Delete, Result = "Delete")]
        [TestCase(StringAndLanguageAttributeFileAccess.Read | StringAndLanguageAttributeFileAccess.Write | StringAndLanguageAttributeFileAccess.Delete, Result = "ReadWrite, Delete")]
        [TestCase(StringAndLanguageAttributeFileAccess.ReadWrite, Result = "ReadWrite")]
        public string ToFormatString_EnumValue_PlainStringRepresentation(Enum value)
        {
            return value.ToFormatString(EnumStringRepresentationUsage.ToStringMethod);
        }

        /// <summary>A test function that compares the result of <see cref="EnumExtensions.ToFormatString(Enum, EnumStringRepresentationUsage)"/> with respect to <see cref="EnumStringRepresentationUsage.StringAttribute"/>.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The <see cref="System.String"/> representation of <paramref name="value"/>.</returns>
        [TestCase(Weekdays.Sunday, Result = "Sunday")]
        [TestCase(Weekdays.Monday, Result = "Monday")]
        [TestCase(Weekdays.Tuesday, Result = "Tuesday")]
        [TestCase(StringAttributeWeekdays.Sunday, Result = "Sonntag")]
        [TestCase(StringAttributeWeekdays.Monday, Result = "Montag")]
        [TestCase(StringAttributeWeekdays.Tuesday, Result = "Dienstag")]
        [TestCase(LanguageAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(LanguageAttributeWeekdays.Monday, Result = "Monday")]
        [TestCase(LanguageAttributeWeekdays.Tuesday, Result = "Tuesday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Sunday, Result = "Sonntag")]
        [TestCase(StringAndLanguageAttributeWeekdays.Monday, Result = "Montag")]
        [TestCase(StringAndLanguageAttributeWeekdays.Tuesday, Result = "Dienstag")]
        [TestCase(PartialAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(PartialAttributeWeekdays.Monday, Result = "Montag")]
        [TestCase(PartialAttributeWeekdays.Tuesday, Result = "Dienstag")]
        [TestCase(FileAccess.Read, Result = "Read")]
        [TestCase(FileAccess.Write, Result = "Write")]
        [TestCase(FileAccess.ReadWrite, Result = "ReadWrite")]
        [TestCase(StringAttributeFileAccess.Read, Result = "Lesen")]
        [TestCase(StringAttributeFileAccess.Write, Result = "Schreiben")]
        [TestCase(StringAttributeFileAccess.Delete, Result = "Löschen")]
        [TestCase(StringAttributeFileAccess.Read | StringAttributeFileAccess.Write | StringAttributeFileAccess.Delete, Result = "Lesen oder schreiben,Löschen")]
        [TestCase(StringAndLanguageAttributeFileAccess.ReadWrite, Result = "Lesen oder schreiben")]
        [TestCase(StringAndLanguageAttributeFileAccess.Read, Result = "Lesen")]
        [TestCase(StringAndLanguageAttributeFileAccess.Write, Result = "Schreiben")]
        [TestCase(StringAndLanguageAttributeFileAccess.Delete, Result = "Löschen")]
        [TestCase(StringAndLanguageAttributeFileAccess.Read | StringAndLanguageAttributeFileAccess.Write | StringAndLanguageAttributeFileAccess.Delete, Result = "Lesen oder schreiben,Löschen")]
        [TestCase(StringAndLanguageAttributeFileAccess.ReadWrite, Result = "Lesen oder schreiben")]
        public string ToFormatString_EnumValue_StringAttribute(Enum value)
        {
            return value.ToFormatString(EnumStringRepresentationUsage.StringAttribute);
        }

        /// <summary>A test function that compares the result of <see cref="EnumExtensions.ToFormatString(Enum, EnumStringRepresentationUsage)"/> with respect to <see cref="EnumStringRepresentationUsage.LanguageStringAttribute"/>.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The <see cref="System.String"/> representation of <paramref name="value"/>.</returns>
        [TestCase(Weekdays.Sunday, Result = "Sunday")]
        [TestCase(Weekdays.Monday, Result = "Monday")]
        [TestCase(Weekdays.Tuesday, Result = "Tuesday")]
        [TestCase(StringAttributeWeekdays.Sunday, Result = "Sonntag")]
        [TestCase(StringAttributeWeekdays.Monday, Result = "Montag")]
        [TestCase(StringAttributeWeekdays.Tuesday, Result = "Dienstag")]
        [TestCase(LanguageAttributeWeekdays.Sunday, Result = "ResourceSunday")]
        [TestCase(LanguageAttributeWeekdays.Monday, Result = "ResourceMonday")]
        [TestCase(LanguageAttributeWeekdays.Tuesday, Result = "ResourceTuesday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Sunday, Result = "ResourceSunday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Monday, Result = "ResourceMonday")]
        [TestCase(StringAndLanguageAttributeWeekdays.Tuesday, Result = "ResourceTuesday")]
        [TestCase(PartialAttributeWeekdays.Sunday, Result = "Sunday")]
        [TestCase(PartialAttributeWeekdays.Monday, Result = "ResourceMonday")]
        [TestCase(PartialAttributeWeekdays.Tuesday, Result = "Dienstag")]
        [TestCase(FileAccess.Read, Result = "Read")]
        [TestCase(FileAccess.Write, Result = "Write")]
        [TestCase(FileAccess.ReadWrite, Result = "ReadWrite")]
        [TestCase(StringAttributeFileAccess.Read, Result = "Lesen")]
        [TestCase(StringAttributeFileAccess.Write, Result = "Schreiben")]
        [TestCase(StringAttributeFileAccess.ReadWrite, Result = "Lesen oder schreiben")]
        [TestCase(StringAndLanguageAttributeFileAccess.ReadWrite, Result = "ResourceReadWrite")]
        [TestCase(StringAndLanguageAttributeFileAccess.Read, Result = "ResourceRead")]
        [TestCase(StringAndLanguageAttributeFileAccess.Write, Result = "ResourceWrite")]
        [TestCase(StringAndLanguageAttributeFileAccess.Delete, Result = "ResourceDelete")]
        [TestCase(StringAndLanguageAttributeFileAccess.Read | StringAndLanguageAttributeFileAccess.Write | StringAndLanguageAttributeFileAccess.Delete, Result = "ResourceReadWrite,ResourceDelete")]
        [TestCase(StringAndLanguageAttributeFileAccess.ReadWrite, Result = "ResourceReadWrite")]
        public string ToFormatString_EnumValue_LanguageStringAttribute(Enum value)
        {
            return value.ToFormatString(EnumStringRepresentationUsage.LanguageStringAttribute);
        }

        /// <summary>A test function that compares the result of <see cref="EnumExtensions.GetDescription(Enum)"/>.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The descrption of <paramref name="value"/> in its <see cref="System.String"/> representation; perhaps <see cref="String.Empty"/>.</returns>
        [TestCase(PartialAttributeWeekdays.Tuesday, Result = "A simple description")]
        [TestCase(PartialAttributeWeekdays.Monday, Result = "ResourceDescription")]
        [TestCase(Weekdays.Sunday, Result = "")]
        public string GetDescription_EnumValue_Description(Enum value)
        {
            return value.GetDescription();
        }
    }
}