﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents.Utilities;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>A sample enumeration containing several weekdays, no attributes are used.
    /// </summary>
    internal enum Weekdays
    {
        Sunday,
        Monday,
        Tuesday
    }

    /// <summary>A sample enumeration containing several weekdays, <see cref="StringAttribute"/> annotations are used.
    /// </summary>
    internal enum StringAttributeWeekdays
    {
        [String("Sonntag")]
        Sunday,

        [String("Montag")]
        Monday,

        [String("Dienstag")]
        Tuesday
    }

    /// <summary>A sample enumeration containing several weekdays, <see cref="LanguageStringAttribute"/> annotations are used.
    /// </summary>
    [LanguageResource("Dodoni.BasicComponents.Utilities.EnumTestResources")]
    internal enum LanguageAttributeWeekdays
    {
        [LanguageString("Sunday")]
        Sunday,

        [LanguageString("Monday")]
        Monday,

        [LanguageString("Tuesday")]
        Tuesday
    }


    /// <summary>A sample enumertion containing several weekdays, <see cref="StringAttribute"/> and <see cref="LanguageStringAttribute"/> annotations are used.
    /// </summary>
    [LanguageResource("Dodoni.BasicComponents.Utilities.EnumTestResources")]
    internal enum StringAndLanguageAttributeWeekdays
    {
        [LanguageString("Sunday")]
        [String("Sonntag")]
        Sunday,

        [String("Montag")]
        [LanguageString("Monday")]
        Monday,

        [LanguageString("Tuesday")]
        [String("Dienstag")]
        Tuesday
    }

    /// <summary>A sample enumertion containing several weekdays, <see cref="StringAttribute"/> and <see cref="LanguageStringAttribute"/> annotations are used.
    /// </summary>
    [LanguageResource("Dodoni.BasicComponents.Utilities.EnumTestResources")]
    internal enum PartialAttributeWeekdays
    {
        /// <summary>The 'Sunday', no annotation.
        /// </summary>
        Sunday,

        /// <summary>The 'Monday', with <see cref="StringAttribute"/> and <see cref="LanguageStringAttribute"/> annotations.
        /// </summary>
        [String("Montag")]
        [LanguageString("Monday")]
        [LanguageDescription("MondayDescription")]
        [Description("This attribute should be ignored, 'LanguageDescription' will be prefered.")]
        Monday,

        /// <summary>The 'Tuesday', with <see cref="StringAttribute"/> and <see cref="DescriptionAttribute"/> annotation.
        /// </summary>
        [String("Dienstag")]
        [Description("A simple description")]
        Tuesday
    }

    /// <summary>A sample enumeration (bit-field) representing a specific file access, no further attributes are used.
    /// </summary>
    [Flags]
    internal enum FileAccess
    {
        Read = 1,
        Write = 2,
        Delete = 4,
        ReadWrite = Read | Write
    }

    /// <summary>A sample enumeration (bit-field) representing a specific file access, <see cref="StringAttribute"/> annotations are used.
    /// </summary>
    [Flags]
    internal enum StringAttributeFileAccess
    {
        [String("Lesen")]
        Read = 1,

        [String("Schreiben")]
        Write = 2,

        [String("Löschen")]
        Delete = 4,

        [String("Lesen oder schreiben")]
        ReadWrite = Read | Write
    }

    /// <summary>A sample enumeration (bit-field) representing a specific file access, <see cref="LanguageStringAttribute"/> annotations are used.
    /// </summary>
    [LanguageResource("Dodoni.BasicComponents.Utilities.EnumTestResources")]
    [Flags]
    internal enum LanguageAttributeFileAccess
    {
        [LanguageString("Read")]
        Read = 1,

        [LanguageString("Write")]
        Write = 2,

        [LanguageString("Delete")]
        Delete = 4,

        [LanguageString("ReadWrite")]
        ReadWrite = Read | Write
    }


    /// <summary>A sample enumeration (bit-field) representing a specific file access,  <see cref="StringAttribute"/> and <see cref="LanguageStringAttribute"/> annotations are used.
    /// </summary>
    [LanguageResource("Dodoni.BasicComponents.Utilities.EnumTestResources")]
    [Flags]
    internal enum StringAndLanguageAttributeFileAccess
    {
        [String("Lesen")]
        [LanguageString("Read")]
        Read = 1,

        [LanguageString("Write")]
        [String("Schreiben")]
        Write = 2,

        [LanguageString("Delete")]
        [String("Löschen")]
        Delete = 4,

        [LanguageString("ReadWrite")]
        [String("Lesen oder schreiben")]
        ReadWrite = Read | Write
    }
}