﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;

using Dodoni.BasicComponents.Utilities;

using NUnit.Framework;

namespace Dodoni.BasicComponents.Utilities
{
    /// <summary>A unit test class for <see cref="Extensions"/>.
    /// </summary>
    [TestFixture]
    public class ExtensionsTests
    {
        /// <summary>A test function that compares the result of <see cref="Extensions.GetRawDistanceTo(DateTime, DateTime)"/>.
        /// </summary>        
        [Test]
        public void GetRawDistanceTo__From_01_01_2010_To_01_01_2020__10()
        {
            DateTime startDate = new DateTime(2010, 1, 1);
            DateTime endDate = new DateTime(2020, 1, 1);

            Assert.AreEqual(10, Extensions.GetRawDistanceTo(startDate, endDate), 1E-10);
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.GetRawDistanceTo(DateTime, DateTime)"/>.
        /// </summary>        
        [Test]
        public void GetRawDistanceTo__From_01_01_2010_To_30_06_2020__10_5()
        {
            DateTime startDate = new DateTime(2010, 1, 1);
            DateTime endDate = new DateTime(2020, 6, 30);

            Assert.AreEqual(10.5, Extensions.GetRawDistanceTo(startDate, endDate), 1E-2);
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.GetRawDistanceTo(DateTime, DateTime)"/>.
        /// </summary>        
        [Test]
        public void GetRawDistanceTo__From_07_05_2009_To_01_04_2015__5_897()
        {
            DateTime startDate = new DateTime(2009, 5, 7);
            DateTime endDate = new DateTime(2015, 4, 1);

            // 5Y 10M 23d
            Assert.AreEqual(5.0 + 10.0 / 12.0 + (23.0 / 30.0) / 12.0, Extensions.GetRawDistanceTo(startDate, endDate), 1E-2);
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.GetRawDistanceTo(DateTime, DateTime)"/>.
        /// </summary>        
        [Test]
        public void GetRawDistanceTo__From_01_04_2015_To_07_05_2009__5_897()
        {
            DateTime startDate = new DateTime(2015, 4, 1);
            DateTime endDate = new DateTime(2009, 5, 7);

            // absolute( -[5Y 10M 23d] ) 
            Assert.AreEqual(5.0 + 10.0 / 12.0 + (23.0 / 30.0) / 12.0, Extensions.GetRawDistanceTo(startDate, endDate), 1E-2);
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.GetRawDistanceTo(DateTime, DateTime)"/>.
        /// </summary>        
        [Test]
        public void GetRawDistanceTo__From_09_10_2003_To_23_08_2014__10_872()
        {
            DateTime startDate = new DateTime(2003, 10, 9);
            DateTime endDate = new DateTime(2014, 8, 23);

            // 10Y 9M 44d
            Assert.AreEqual(10.0 + 9.0 / 12.0 + (44.0 / 30.0) / 12.0, Extensions.GetRawDistanceTo(startDate, endDate), 1E-2);

            // 10Y 10M 14d
            Assert.AreEqual(10.0 + 10.0 / 12.0 + (14.0 / 30.0) / 12.0, Extensions.GetRawDistanceTo(startDate, endDate), 1E-2);
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.BinarySearch&lt;T&gt;(IList&lt;T&gt;,T)"/>.
        /// </summary>        
        [Test]
        public void BinarySearch__0To99And11__11()
        {
            IList<double> sortedListOfNumbers = Enumerable.Range(0, 100).Select(x => (double)x).ToList();
            Assert.AreEqual(11, Extensions.BinarySearch<double>(sortedListOfNumbers, 11.0));
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.BinarySearch&lt;T&gt;(IList&lt;T&gt;,T)"/>.
        /// </summary>        
        [Test]
        public void BinarySearch__0To99And99__99()
        {
            IList<double> sortedListOfNumbers = Enumerable.Range(0, 100).Select(x => (double)x).ToList();
            Assert.AreEqual(99, Extensions.BinarySearch<double>(sortedListOfNumbers, 99.0));
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.BinarySearch&lt;T&gt;(IList&lt;T&gt;,T)"/>.
        /// </summary>        
        [Test]
        public void BinarySearch__0To99And100__BitwiseComplementOf100()
        {
            IList<double> sortedListOfNumbers = Enumerable.Range(0, 100).Select(x => (double)x).ToList();
            Assert.AreEqual(~100, Extensions.BinarySearch<double>(sortedListOfNumbers, 100.0));
        }


        /// <summary>A test function that compares the result of <see cref="Extensions.BinarySearch&lt;T&gt;(IList&lt;T&gt;,T)"/>.
        /// </summary>        
        [Test]
        public void BinarySearch__0To99And0__0()
        {
            IList<double> sortedListOfNumbers = Enumerable.Range(0, 100).Select(x => (double)x).ToList();
            Assert.AreEqual(0, Extensions.BinarySearch<double>(sortedListOfNumbers, 0.0));
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.BinarySearch&lt;T&gt;(IList&lt;T&gt;,T)"/>.
        /// </summary>        
        [Test]
        public void BinarySearch__0To99And0_5__BitwiseComplementOf1()
        {
            IList<double> sortedListOfNumbers = Enumerable.Range(0, 100).Select(x => (double)x).ToList();
            Assert.AreEqual(~1, Extensions.BinarySearch<double>(sortedListOfNumbers, 0.5));
        }

        /// <summary>A test function that compares the result of <see cref="Extensions.BinarySearch&lt;T&gt;(IList&lt;T&gt;,T)"/>.
        /// </summary>        
        [Test]
        public void BinarySearch__0To99AndMinus2_5__Minus1()
        {
            List<double> sortedListOfNumbers = Enumerable.Range(0, 100).Select(x => (double)x).ToList();
            
            Assert.AreEqual(sortedListOfNumbers.BinarySearch(-2.5), Extensions.BinarySearch<double>(sortedListOfNumbers, -2.5));
            Assert.AreEqual(-1, Extensions.BinarySearch<double>(sortedListOfNumbers, -2.5));
        }
    }
}