﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    public partial class LapackEigenvalues
    {
        public interface ISingularValueDecomposition
        {
            /// <summary>Reduces a general band matrix to bidiagonal form
            /// </summary>
            /// <param name="vect">The vect.</param>
            /// <param name="m">The m.</param>
            /// <param name="n">The n.</param>
            /// <param name="ncc">The NCC.</param>
            /// <param name="kl">The kl.</param>
            /// <param name="ku">The ku.</param>
            /// <param name="ab">The ab.</param>
            /// <param name="ldab">The ldab.</param>
            /// <param name="d">The d.</param>
            /// <param name="e">The e.</param>
            /// <param name="q">The q.</param>
            /// <param name="ldq">The LDQ.</param>
            /// <param name="pt">The pt.</param>
            /// <param name="ldpt">The LDPT.</param>
            /// <param name="c">The c.</param>
            /// <param name="ldc">The LDC.</param>
            /// <param name="work">The work.</param>
            void dgbbrd(char vect, int m, int n, int ncc, int kl, int ku, double[] ab, int ldab, double[] d, double[] e, double[] q, int ldq, double[] pt, int ldpt, double[] c, int ldc, double[] work);

            /// <summary>Computes the singular value decomposition of a general matrix that has been reduced to bidiagonal form.
            /// </summary>
            /// <param name="uplo">The uplo.</param>
            /// <param name="n">The n.</param>
            /// <param name="ncvt">The NCVT.</param>
            /// <param name="nru">The nru.</param>
            /// <param name="ncc">The NCC.</param>
            /// <param name="d">The d.</param>
            /// <param name="e">The e.</param>
            /// <param name="vt">The vt.</param>
            /// <param name="ldvt">The LDVT.</param>
            /// <param name="u">The u.</param>
            /// <param name="ldu">The ldu.</param>
            /// <param name="c">The c.</param>
            /// <param name="ldc">The LDC.</param>
            /// <param name="work">The work.</param>
            void dbdsqr(LAPACK.BidiagonalMatrixType uplo, int n, int ncvt, int nru, int ncc, double[] d, double[] e, double[] vt, int ldvt, double[] u, int ldu, double[] c, int ldc, double[] work);
        }
    }
}