﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Numerics;

using Dodoni.MathLibrary.Basics.LowLevel.Native;

namespace Dodoni.MathLibrary.Basics.LowLevel
{
    /// <summary>Provides routines of the Linear Algebra PACKage, see http://www.netlib.org/lapack/index.html, that are related to Least Square and Eigenvalue calculations.
    /// </summary>
    public partial class LapackEigenvalues
    {
        #region public (readonly) members

        /// <summary>Provides driver routines to solve a specified problem related to Least Square or Eigenvalue calculation.
        /// </summary>
        public readonly IDriverRoutines DriverRoutines = null;

        /// <summary>Provides routines for generalized non-symmetric Eigenvalue problems.
        /// </summary>
        public readonly IGeneralizedNonsymmetricEigenvalueProblems GeneralizedNonSymmetric = null;

        /// <summary>Provides routines for generalized symmetric Eigenvalue problems.
        /// </summary>
        public readonly IGeneralizedSymmetricEigenvalueProblems GeneralizedSymmetric = null;

        /// <summary>Provides routines for non-symmetric Eigenvalue problems.
        /// </summary>
        public readonly INonsymmetricEigenvalueProblems NonSymmetric = new LapackNativeWrapper();

        /// <summary>Provides routines for symmetric Eigenvalue problems.
        /// </summary>
        public readonly ISymmetricEigenvalueProblems Symmetric = null;

        /// <summary>Provides routines for the Singular Value Decomposition of a specified matrix.
        /// </summary>
        public readonly ISingularValueDecomposition SingularValueDecomposition = null;

        /// <summary>Provides routines for the generalized Singular Value Decomposition of two matrices.
        /// </summary>
        public readonly IGeneralizedSingularValueDecomposition GeneralizedSingularValueDecomposition = null;
        #endregion

        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="LapackEigenvalues" /> class.
        /// </summary>
        internal LapackEigenvalues()
        {
        }
        #endregion
    }
}