﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Numerics;
using System.Security;
using System.Runtime.InteropServices;

namespace Dodoni.MathLibrary.Basics.LowLevel.Native
{
    /// <summary>Serves as native wrapper for the Lapack functions, see http://www.netlib.org/lapack/index.html.
    /// </summary>
    internal partial class LapackNativeWrapper : LapackLinearEquations.IMatrixFactorization
    {
        #region private function import

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DGBTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dgbtrf(ref int m, ref int n, ref int kl, ref int ku, IntPtr ab, ref int ldab, IntPtr iPivot, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DGETRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dgetrf(ref int m, ref int n, IntPtr a, ref int lda, IntPtr iPivot, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DGTTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dgttrf(ref int n, IntPtr dl, IntPtr d, IntPtr du, IntPtr du2, IntPtr ipiv, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DPOTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dpotrf(ref char uplo, ref int n, IntPtr a, ref int lda, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DPBTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dpbtrf(ref char uplo, ref int n, ref int kd, IntPtr ab, ref int ldab, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DPPTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dpptrf(ref char uplo, ref int n, IntPtr ap, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DPFTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dpftrf(ref char transr, ref char uplo, ref int n, IntPtr a, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DPTTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dpttrf(ref int n, IntPtr d, IntPtr e, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DSPTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dsptrf(ref char uplo, ref int n, IntPtr ap, IntPtr iPiv, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "DSYTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _dsytrf(ref char uplo, ref int n, IntPtr a, ref int lda, IntPtr iPiv, IntPtr work, ref int lwork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZGETRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zgetrf(ref int m, ref int n, IntPtr a, ref int lda, IntPtr iPivot, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZGBTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zgbtrf(ref int m, ref int n, ref int kl, ref int ku,  IntPtr ab, ref int ldab, IntPtr iPivot, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZGTTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zgttrf(ref int n, IntPtr dl, IntPtr d, IntPtr du, IntPtr du2, IntPtr ipiv, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZPOTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zpotrf(ref char uplo, ref int n, IntPtr a, ref int lda, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZPFTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zpftrf(ref char transr, ref char uplo, ref int n, IntPtr a, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZPPTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zpptrf(ref char uplo, ref int n, IntPtr ap, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZPBTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zpbtrf(ref char uplo, ref int n, ref int kd, IntPtr ab, ref int ldab, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZPTTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zpttrf(ref int n, IntPtr d, IntPtr e, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZSYTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zsytrf(ref char uplo, ref int n, IntPtr a, ref int lda, IntPtr iPiv, IntPtr work, ref int lwork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZHETRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zhetrf(ref char uplo, ref int n, IntPtr a, ref int lda, IntPtr ipiv, IntPtr work, ref int lwork, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZSPTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zsptrf(ref char uplo, ref int n, IntPtr ap, IntPtr iPiv, out int info);

        [DllImport(LapackNativeWrapper.sm_DllName, EntryPoint = "ZHPTRF", CallingConvention = CallingConvention.Cdecl), SuppressUnmanagedCodeSecurity]
        private static extern void _zhptrf(ref char uplo, ref int n, IntPtr ap, IntPtr ipiv, out int info);
        #endregion

        #region IMatrixFactorization Members

        #region double precision methods

        /// <summary>Computes the LU factorization of a general m-by-n matrix, i.e. A = P * L * U, where P is a permutation matrix, L is lower triangular with unit diagonal elements and U is upper triangular.
        /// </summary>
        /// <param name="m">The number of rows of the matrix.</param>
        /// <param name="n">The number of columns of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by L and U, the unit diagonal elements of L are not stored; output.</param>
        /// <param name="iPivot">The min(<paramref name="m"/>,<paramref name="n"/>) pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j] (output).</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>.</param>
        public void dgetrf(int m, int n, double[] a, int[] iPivot, int lda)
        {
            int info;
            unsafe
            {
                fixed (double* aPtr = a)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _dgetrf(ref m, ref n, (IntPtr)aPtr, ref lda, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }

        /// <summary>Computes the LU factorization of a general m-by-n band matrix, i.e. A = P * L * U, where P is a permutation matrix, L is lower triangular with unit diagonal elements and U is upper triangular.
        /// </summary>
        /// <param name="m">The number of rows of the matrix.</param>
        /// <param name="n">The number of columns of the matrix.</param>
        /// <param name="kl">The number of subdiagonals within the band of the input matrix.</param>
        /// <param name="ku">The number of superdiagonals within the band of the input matrix.</param>
        /// <param name="a">The input band matrix in band storage, i.e. of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by L and U. U is stored as an upper triangular band matrix with <paramref name="kl"/> + <paramref name="ku"/> superdiagonals.</param>
        /// <param name="iPivot">The pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j]; must contain at least min(<paramref name="m"/>,<paramref name="n"/>) elements (output).</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, at least 2 * <paramref name="kl"/> + <paramref name="ku"/> + 1.</param>
        public void dgbtrf(int m, int n, int kl, int ku, double[] a, int[] iPivot, int lda)
        {
            int info;
            unsafe
            {
                fixed (double* aPtr = a)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _dgbtrf(ref m, ref n, ref kl, ref ku, (IntPtr)aPtr, ref lda, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }

        /// <summary>Computes the LU factorization of a tridiagonal matrix, i.e. A = P * L * U, where P is a permutation matrix, L is lower bidiagonal with unit diagonal elements and U is upper triangular
        /// with nonzeroes in only the main diagonal and first two superdiagonals.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="dl">The (<paramref name="n"/> - 1) subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) multipliers that defines the matrix L from the LU factorization.</param>
        /// <param name="d">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal elements of the upper triangular matrix U from the LU factorization.</param>
        /// <param name="du">The (<paramref name="n"/> - 1) superdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) elements of the first superdiagonal of U.</param>
        /// <param name="du2">The (<paramref name="n"/> - 2) elements of the second superdiagonal of U (output).</param>
        /// <param name="iPivot">The <paramref name="n"/> pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j].
        /// <paramref name="iPivot"/>[j] is always j or j+1; <paramref name="iPivot"/>[j] = j indicates a row interchange was not required (output).</param>
        /// <remarks>The matrix L has unit diagonal elements and the (<paramref name="n"/> - 1) elements of d1 from the subdiagonal. All other elements of L are zero.</remarks>
        public void dgttrf(int n, double[] dl, double[] d, double[] du, double[] du2, int[] iPivot)
        {
            int info;
            unsafe
            {
                fixed (double* dlPtr = dl, dPTr = d, duPtr = du, du2Ptr = du)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _dgttrf(ref n, (IntPtr)dlPtr, (IntPtr)dPTr, (IntPtr)duPtr, (IntPtr)du2Ptr, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }

        /// <summary>Computes the Cholesky decomposition of a symmetric positive-definite matrix, i.e. A = U' * U or A = L * L', where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda" />; <paramref name="n" />); overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        /// <param name="lda">The leading dimension of <paramref name="a" />.</param>
        public void dpotrf(int n, double[] a, BLAS.TriangularMatrixType factorizationType, int lda)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (double* aPtr = a)
                {
                    _dpotrf(ref uplo, ref n, (IntPtr)aPtr, ref lda, out info);
                }
            }
        }

        /// <summary>
        /// Computes the Cholesky factorization of a symmetric positive-definite matrix using the Rectangular Full Packed (RFP) format, i.e.
        /// A = U' * U or A = L * L', where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A in the RFP format, i.e. an array with at least <paramref name="n"/> * (<paramref name="n"/> + 1) / 2 elements; overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        /// <param name="transpose">A value indicating whether <paramref name="a"/> represents matrix A or its transposed.</param>
        public void dpftrf(int n, double[] a, BLAS.TriangularMatrixType factorizationType, BLAS.MatrixTransposeState transpose)
        {
            int info;
            char transr = 'l';
            char uplo = 'l';

            unsafe
            {
                fixed (double* aPtr = a)
                {
                    _dpftrf(ref transr, ref uplo, ref n, (IntPtr)aPtr, out info);
                }
            }
        }

        /// <summary>Computes the Cholesky factorization of a symmetric positive-definite matrix using packed storage, i.e.
        /// A = U' * U or A = L * L', where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="aPacked">Either the upper or lower triangular part of matrix A in packed storage, i.e. at least <paramref name="n"/> * (<paramref name="n"/> + 1)/2 elements; overwritten by the upper or lower triangular matrix U, L respectively in packed storage.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        public void dpptrf(int n, double[] aPacked, BLAS.TriangularMatrixType factorizationType)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (double* aPackedPtr = aPacked)
                {
                    _dpptrf(ref uplo, ref n, (IntPtr)aPackedPtr, out info);
                }
            }
        }

        /// <summary>Computes the Cholesky factorization of a symmetric positive-definite band matrix, i.e. i.e. A = U' * U or A = L * L', where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="dCount">The number of superdiagonals or subdiagonals in the input matrix.</param>
        /// <param name="a">Either the upper or lower triangular part of the input matrix in band storage of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least <paramref name="dCount"/> + 1.</param>
        public void dpbtrf(int n, int dCount, double[] a, BLAS.TriangularMatrixType factorizationType, int lda)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (double* aPtr = a)
                {
                    _dpbtrf(ref uplo, ref n, ref dCount, (IntPtr)aPtr, ref lda, out info);
                }
            }
        }

        /// <summary>
        /// Computes the factorization of a symmetric positive-definite tridiagonal matrix, i.e. A = L * D * L' or A = U' * D * U, where D is diagonal, L is unit lower bidiagonal and U is unit upper bidiagonal.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="diagonalElements">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal elements of the diagonal matrix D.</param>
        /// <param name="e">The <paramref name="n"/> - 1 subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) off-diagonal elements of the unit bidiagonal factor L or U from the factorization.</param>
        public void dpttrf(int n, double[] diagonalElements, double[] e)
        {
            int info;
            unsafe
            {
                fixed (double* diagonalElementsPtr = diagonalElements, ePtr = e)
                {
                    _dpttrf(ref n, (IntPtr)diagonalElementsPtr, (IntPtr)ePtr, out info);
                }
            }
        }

        /// <summary>Computes the Bunch-Kaufman factorization of a symmetric matrix, i.e. A = P * U * D * U' * P' or A = P * L * D * L' * P', where P is a permutation matrix, U and L are upper and
        /// lower triangular matrices with unit diagonal and D is a symmetric block-diagonal matrix with 1-by-1 and 2-by-2 diagonal blocks. U and L have 2-by-2 unit diagonal blocks corresponding to the 2-by-2 blocks of D.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The upper or the lower triangular part of the input matrix of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by details of the block-diagonal matrix D and the multiplies used to obtain the factor U (or L).</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least <paramref name="n"/>.</param>
        /// <param name="iPivot">Contains details of the interchanges an the block structure of D, at least <paramref name="n"/> elements (output).</param>
        /// <param name="work">A workspace array of length at least <paramref name="workLength"/>.</param>
        /// <param name="workLength">The length of the workspace array <paramref name="work"/>, at least <paramref name="n"/>.</param>
        public void dsytrf(int n, double[] a, BLAS.TriangularMatrixType factorizationType, int lda, int[] iPivot, double[] work, int workLength)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (double* aPtr = a, workPtr = work)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _dsytrf(ref uplo, ref n, (IntPtr)aPtr, ref lda, (IntPtr)iPivotPtr, (IntPtr)workPtr, ref workLength, out info);
                    }
                }
            }
        }

        /// <summary>Computes the Bunch-Kaufman factorization of a symmetric matrix using packed storage, i.e. A = P * U * D * U' * P' or A = P * L * D * L' * P', where P is a permutation matrix, U and L are upper and
        /// lower triangular matrices with unit diagonal and D is a symmetric block-diagonal matrix with 1-by-1 and 2-by-2 diagonal blocks. U and L have 2-by-2 unit diagonal blocks corresponding to the 2-by-2 blocks of D.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="aPacked">Either the upper or lower triangular part of matrix A in packed storage, i.e. at least <paramref name="n"/> * (<paramref name="n"/> + 1)/2 elements; overwritten by details of the block-diagonal matrix D and the multiplies used to obtain the factor U (or L).</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="iPivot">Contains details of the interchanges an the block structure of D, at least <paramref name="n"/> elements (output).</param>
        public void dsptrf(int n, double[] aPacked, BLAS.TriangularMatrixType factorizationType, int[] iPivot)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (double* aPackagedPtr = aPacked)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _dsptrf(ref uplo, ref n, (IntPtr)aPackagedPtr, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }
        #endregion

        #region (double precision) complex methods

        /// <summary>Computes the LU factorization of a general m-by-n matrix, i.e. A = P * L * U, where P is a permutation matrix, L is lower triangular with unit diagonal elements and U is upper triangular.
        /// </summary>
        /// <param name="m">The number of rows of the matrix.</param>
        /// <param name="n">The number of columns of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by 'L' and 'U', the unit diagonal elements of 'L' are not stored; output.</param>
        /// <param name="iPivot">The min(<paramref name="m"/>,<paramref name="n"/>) pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j] (output).</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>.</param>
        public void zgetrf(int m, int n, Complex[] a, int[] iPivot, int lda)
        {
            int info;
            unsafe
            {
                fixed (Complex* aPtr = a)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zgetrf(ref m, ref n, (IntPtr)aPtr, ref lda, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }

        /// <summary>Computes the LU factorization of a general m-by-n band matrix, i.e. A = P * L * U, where P is a permutation matrix, L is lower triangular with unit diagonal elements and U is upper triangular.
        /// </summary>
        /// <param name="m">The number of rows of the matrix.</param>
        /// <param name="n">The number of columns of the matrix.</param>
        /// <param name="kl">The number of subdiagonals within the band of the input matrix.</param>
        /// <param name="ku">The number of superdiagonals within the band of the input matrix.</param>
        /// <param name="a">The input band matrix in band storage, i.e. of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by L and U. U is stored as an upper triangular band matrix with <paramref name="kl"/> + <paramref name="ku"/> superdiagonals.</param>
        /// <param name="iPivot">The pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j]; must contain at least min(<paramref name="m"/>,<paramref name="n"/>) elements (output).</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, at least 2 * <paramref name="kl"/> + <paramref name="ku"/> + 1.</param>
        public void zgbtrf(int m, int n, int kl, int ku, Complex[] a, int[] iPivot, int lda)
        {
            int info;
            unsafe
            {
                fixed (Complex* aPtr = a)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zgbtrf(ref m, ref n, ref kl, ref ku, (IntPtr)aPtr, ref lda, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }

        /// <summary>
        /// Computes the LU factorization of a tridiagonal matrix, i.e. A = P * L * U, where P is a permutation matrix, L is lower bidiagonal with unit diagonal elements and U is upper triangular
        /// with nonzeroes in only the main diagonal and first two superdiagonals.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="dl">The (<paramref name="n"/> - 1) subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) multipliers that defines the matrix L from the LU factorization.</param>
        /// <param name="d">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal elements of the upper triangular matrix U from the LU factorization.</param>
        /// <param name="du">The (<paramref name="n"/> - 1) superdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) elements of the first superdiagonal of U.</param>
        /// <param name="du2">The (<paramref name="n"/> - 2) elements of the second superdiagonal of U (output).</param>
        /// <param name="iPivot">The <paramref name="n"/> pivot indices, i.e. row j was interchanged with row <paramref name="iPivot"/>[j].
        /// <paramref name="iPivot"/>[j] is always j or j+1; <paramref name="iPivot"/>[j] = j indicates a row interchange was not required (output).</param>
        /// <remarks>The matrix L has unit diagonal elements and the (<paramref name="n"/> - 1) elements of d1 from the subdiagonal. All other elements of L are zero.</remarks>
        public void zgttrf(int n, Complex[] dl, Complex[] d, Complex[] du, Complex[] du2, int[] iPivot)
        {
            int info;
            unsafe
            {
                fixed (Complex* dlPtr = dl, dPtr = d, duPtr = du,du2Ptr = du)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zgttrf(ref n, (IntPtr)dlPtr, (IntPtr)dPtr, (IntPtr)duPtr, (IntPtr)du2Ptr, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }
       
        /// <summary>Computes the Cholesky decomposition of a Hermitian positive-definite matrix, i.e. A = conj(U') * U or A = L * conj(L'), where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A supplied column-by-column of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>.</param>
        public void zpotrf(int n, Complex[] a, BLAS.TriangularMatrixType factorizationType, int lda)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (Complex* aPtr = a)
                {
                    _zpotrf(ref uplo, ref n, (IntPtr)aPtr, ref lda, out info);
                }
            }
        }

        /// <summary>
        /// Computes the Cholesky factorization of a Hermitian positive-definite matrix using the Rectangular Full Packed (RFP) format, i.e.
        /// A = conj(U') * U or A = L * conj(L'), where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The matrix A in the RFP format, i.e. an array with at least <paramref name="n"/> * (<paramref name="n"/> + 1) / 2 elements; overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        /// <param name="transpose">A value indicating whether <paramref name="a"/> represents matrix A or its transposed.</param>
        public void zpftrf(int n, Complex[] a, BLAS.TriangularMatrixType factorizationType, BLAS.MatrixTransposeState transpose)
        {
            int info;
            char transr = 'l';
            char uplo = 'l';

            unsafe
            {
                fixed (Complex* aPtr = a)
                {
                    _zpftrf(ref transr, ref uplo, ref n, (IntPtr)aPtr, out info);
                }
            }
        }

        /// <summary>
        /// Computes the Cholesky factorization of a Hermitian positive-definite matrix using packed storage, i.e.
        /// A = conj(U') * U or A = L * conj(L'), where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="aPacked">Either the upper or lower triangular part of matrix A in packed storage, i.e. at least <paramref name="n" /> * (<paramref name="n" /> + 1)/2 elements; overwritten by the upper or lower triangular matrix U, L respectively in packed storage.</param>
        /// <param name="factorizationType">A value indicating whether the upper or lower triangular part of matrix A is stored and how matrix A is factored.</param>
        public void zpptrf(int n, Complex[] aPacked, BLAS.TriangularMatrixType factorizationType)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (Complex* aPackedPtr = aPacked)
                {
                    _zpptrf(ref uplo, ref n, (IntPtr)aPackedPtr, out info);
                }
            }
        }

        /// <summary>
        /// Computes the Cholesky factorization of a Hermitian positive-definite band matrix, i.e. i.e. A = conj(U') * U or A = L * conj(L'), where L is a lower triangular matrix and U is upper triangular.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="dCount">The number of superdiagonals or subdiagonals in the input matrix.</param>
        /// <param name="a">Either the upper or lower triangular part of the input matrix in band storage of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by the upper or lower triangular matrix U, L respectively.</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least <paramref name="dCount"/> + 1.</param>
        public void zpbtrf(int n, int dCount, Complex[] a, BLAS.TriangularMatrixType factorizationType, int lda)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (Complex* aPtr = a)
                {
                    _zpbtrf(ref uplo, ref n, ref dCount, (IntPtr)aPtr, ref lda, out info);
                }
            }
        }

        /// <summary>
        /// Computes the factorization of a Hermitian positive-definite tridiagonal matrix, i.e. A = L * D * conj(L') or A = conj(U') * D * U, where D is diagonal, L is unit lower bidiagonal and U is unit upper bidiagonal.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="diagonalElements">The <paramref name="n"/> diagonal elements of the input matrix; overwritten by the <paramref name="n"/> diagonal elements of the diagonal matrix D.</param>
        /// <param name="e">The <paramref name="n"/> - 1 subdiagonal elements of the input matrix; overwritten by the (<paramref name="n"/> - 1) off-diagonal elements of the unit bidiagonal factor L or U from the factorization.</param>
        public void zpttrf(int n, Complex[] diagonalElements, Complex[] e)
        {
            int info;
            unsafe
            {
                fixed (Complex* diagonalElementsPtr = diagonalElements, ePtr = e)
                {
                    _zpttrf(ref n, (IntPtr)diagonalElementsPtr, (IntPtr)ePtr, out info);
                }
            }
        }

        /// <summary>
        /// Computes the Bunch-Kaufman factorization of a symmetric matrix, i.e. A = P * U * D * U' * P' or A = P * L * D * L' * P', where P is a permutation matrix, U and L are upper and
        /// lower triangular matrices with unit diagonal and D is a symmetric block-diagonal matrix with 1-by-1 and 2-by-2 diagonal blocks. U and L have 2-by-2 unit diagonal blocks corresponding to the 2-by-2 blocks of D.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The upper or the lower triangular part of the input matrix of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by details of the block-diagonal matrix D and the multiplies used to obtain the factor U (or L).</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least <paramref name="n"/>.</param>
        /// <param name="iPivot">Contains details of the interchanges an the block structure of D, at least <paramref name="n"/> elements (output).</param>
        /// <param name="work">A workspace array of length at least <paramref name="workLength"/>.</param>
        /// <param name="workLength">The length of the workspace array <paramref name="work"/>, at least <paramref name="n"/>.</param>
        public void zsytrf(int n, Complex[] a, BLAS.TriangularMatrixType factorizationType, int lda, int[] iPivot, Complex[] work, int workLength)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (Complex* aPtr = a, workPtr = work)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zsytrf(ref uplo, ref n, (IntPtr)aPtr, ref lda, (IntPtr)iPivotPtr, (IntPtr)workPtr, ref workLength, out info);
                    }
                }
            }

        }

        /// <summary>
        /// Computes the Bunch-Kaufman factorization of a complex Hermitian matrix, i.e. A = P * U * D * conj(U') * conj(P') or A = P * L * D * conj(L') * conj(P'), where P is a permutation matrix, U and L are upper and
        /// lower triangular matrices with unit diagonal and D is a symmetric block-diagonal matrix with 1-by-1 and 2-by-2 diagonal blocks. U and L have 2-by-2 unit diagonal blocks corresponding to the 2-by-2 blocks of D.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="a">The upper or the lower triangular part of the input matrix of dimension (<paramref name="lda"/>; <paramref name="n"/>); overwritten by details of the block-diagonal matrix D and the multiplies used to obtain the factor U (or L).</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="lda">The leading dimension of <paramref name="a"/>, i.e. at least <paramref name="n"/>.</param>
        /// <param name="iPivot">Contains details of the interchanges an the block structure of D, at least <paramref name="n"/> elements (output).</param>
        /// <param name="work">A workspace array of length at least <paramref name="workLength"/>.</param>
        /// <param name="workLength">The length of the workspace array <paramref name="work"/>, at least <paramref name="n"/>.</param>
        public void zhetrf(int n, Complex[] a, BLAS.TriangularMatrixType factorizationType, int lda, int[] iPivot, Complex[] work, int workLength)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (Complex* aPtr = a, workPtr = work)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zhetrf(ref uplo,ref n, (IntPtr) aPtr,ref lda, (IntPtr)iPivotPtr,(IntPtr) workPtr,ref workLength, out info);
                    }
                }
            }
        }

        /// <summary>
        /// Computes the Bunch-Kaufman factorization of a symmetric matrix using packed storage, i.e. A = P * U * D * U' * P' or A = P * L * D * L' * P', where P is a permutation matrix, U and L are upper and
        /// lower triangular matrices with unit diagonal and D is a symmetric block-diagonal matrix with 1-by-1 and 2-by-2 diagonal blocks. U and L have 2-by-2 unit diagonal blocks corresponding to the 2-by-2 blocks of D.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="aPacked">Either the upper or lower triangular part of matrix A in packed storage, i.e. at least <paramref name="n"/> * (<paramref name="n"/> + 1)/2 elements; overwritten by details of the block-diagonal matrix D and the multiplies used to obtain the factor U (or L).</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="iPivot">Contains details of the interchanges an the block structure of D, at least <paramref name="n"/> elements (output).</param>
        public void zsptrf(int n, Complex[] aPacked, BLAS.TriangularMatrixType factorizationType, int[] iPivot)
        {
            int info;
            char uplo = 'l';
            unsafe
            {
                fixed (Complex* aPackagedPtr = aPacked)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zsptrf(ref uplo, ref n, (IntPtr)aPackagedPtr, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }

        /// <summary>
        /// Computes the Bunch-Kaufman factorization of a Hermitian matrix using packed storage, i.e. A = P * U * D * conj(U') * conj(P') or A = P * L * D * conj(L') * conj(P'), where P is a permutation matrix, U and L are upper and
        /// lower triangular matrices with unit diagonal and D is a symmetric block-diagonal matrix with 1-by-1 and 2-by-2 diagonal blocks. U and L have 2-by-2 unit diagonal blocks corresponding to the 2-by-2 blocks of D.
        /// </summary>
        /// <param name="n">The order of the matrix.</param>
        /// <param name="aPacked">Either the upper or lower triangular part of matrix A in packed storage, i.e. at least <paramref name="n"/> * (<paramref name="n"/> + 1)/2 elements; overwritten by details of the block-diagonal matrix D and the multiplies used to obtain the factor U (or L).</param>
        /// <param name="factorizationType">Type of the factorization.</param>
        /// <param name="iPivot">Contains details of the interchanges an the block structure of D, at least <paramref name="n"/> elements (output).</param>
        public void zhptrf(int n, Complex[] aPacked, BLAS.TriangularMatrixType factorizationType, int[] iPivot)
        {
            int info;
            char uplo = 'l';

            unsafe
            {
                fixed (Complex* aPackedPtr = aPacked)
                {
                    fixed (int* iPivotPtr = iPivot)
                    {
                        _zhptrf(ref uplo, ref n, (IntPtr)aPackedPtr, (IntPtr)iPivotPtr, out info);
                    }
                }
            }
        }
        #endregion

        #endregion
    }
}