﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;

namespace Dodoni.MathLibrary
{
    public static partial class Extensions
    {
        /// <summary>Copies all the elements of the current (real-valued) matrix to the specified two-dimensional Array.
        /// </summary>
        /// <param name="matrix">The <see cref="IMatrix"/> object.</param>
        /// <param name="array">The array, the first null-based index is the row, the second null-based
        /// index represents the column (output).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="matrix"/> is <c>null</c>.</exception>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="array"/> contains less rows/columns than <paramref name="matrix"/>.</exception>
        public static void CopyTo(this IMatrix matrix, double[,] array)
        {
            if (matrix == null)
            {
                throw new ArgumentNullException("matrix");
            }
            for (int j = 0; j < matrix.RowCount; j++)
            {
                for (int k = 0; k < matrix.ColumnCount; k++)
                {
                    array[j, k] = matrix[j, k];
                }
            }
        }

        /// <summary>Copies all the elements of the current (real-valued) matrix to the specified two-dimensional Array.
        /// </summary>
        /// <param name="matrix">The <see cref="IMatrix"/> object.</param>
        /// <param name="array">The array, the first null-based index is the row, the second null-based
        /// index represents the column (output).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="matrix"/> is <c>null</c>.</exception>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="array"/> contains less rows/columns than <paramref name="matrix"/>.</exception>
        public static void CopyTo(this IMatrix matrix, object[,] array)
        {
            if (matrix == null)
            {
                throw new ArgumentNullException("matrix");
            }
            for (int j = 0; j < matrix.RowCount; j++)
            {
                for (int k = 0; k < matrix.ColumnCount; k++)
                {
                    array[j, k] = matrix[j, k];
                }
            }
        }

        /// <summary>Copies all the elements of the current (real-valued) matrix to the specified two-dimensional Array.
        /// </summary>
        /// <param name="matrix">The <see cref="IMatrix"/> object.</param>
        /// <param name="array">The array, the first null-based index is the row, the second null-based
        /// index represents the column (output).</param>
        /// <exception cref="ArgumentNullException">Thrown, if <paramref name="matrix"/> is <c>null</c>.</exception>
        /// <exception cref="IndexOutOfRangeException">Thrown, if <paramref name="array"/> contains less rows/columns than <paramref name="matrix"/>.</exception>
        [CLSCompliant(false)]
        public static void CopyTo(this IMatrix matrix, double[][] array)
        {
            if (matrix == null)
            {
                throw new ArgumentNullException("matrix");
            }
            for (int j = 0; j < matrix.RowCount; j++)
            {
                for (int k = 0; k < matrix.ColumnCount; k++)
                {
                    array[j][k] = matrix[j, k];
                }
            }
        }
    }
}