﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.GridPointCurves
{
    public static partial class GridPointCurve
    {
        /// <summary>Provides several utility methods concerning grid point curve objects.
        /// </summary>
        internal static class Utilities
        {
            /// <summary>Gets the null-based index of the nearest label which less than the specified value and not the last label.
            /// </summary>
            /// <param name="value">The value which should be compared to the <see cref="System.Double"/> representation of the labels.</param>
            /// <param name="doubleLabels">The <see cref="System.Double"/> representation of the labels in ascending order.</param>
            /// <param name="labelCount">The number of <paramref name="doubleLabels"/>.</param>
            /// <returns>The null-based index of the nearest label which is smaller than <paramref name="value"/>. If <paramref name="value"/> is identical to the 
            /// last label the number of labels -2 will returned.</returns>
            /// <exception cref="ArgumentException">Thrown, if <paramref name="value"/> is smaller than the first element of <paramref name="doubleLabels"/> or greater than 
            /// the last (i.e. (<paramref name="labelCount"/> -1)-th) value of <paramref name="doubleLabels"/>.</exception>
            public static int GetNonLastNearestIndex(double value, double[] doubleLabels, int labelCount)
            {
                if (value < doubleLabels[0])
                {
                    throw new ArgumentException(String.Format(ExceptionMessages.ArgumentOutOfRangeGreaterEqual, value, doubleLabels[0]), "value");
                }
                int nonLastLeftGridPointIndex = Array.BinarySearch<double>(doubleLabels, 0, labelCount, value);
                if (nonLastLeftGridPointIndex < 0)
                {
                    nonLastLeftGridPointIndex = (~nonLastLeftGridPointIndex) - 1;
                    if (nonLastLeftGridPointIndex >= labelCount)  // value is greater than each element of the array
                    {
                        throw new ArgumentException(String.Format(ExceptionMessages.ArgumentOutOfRangeLessEqual, value, doubleLabels[labelCount - 1]), "value");
                    }
                }
                else if (nonLastLeftGridPointIndex == labelCount - 1)
                {
                    return labelCount - 2;
                }
                return nonLastLeftGridPointIndex;
            }
        }
    }
}