/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.MathLibrary
{
    /// <summary>Contains various constants concerning the machine precision, given in some <see cref="System.Double"/> representation.
    /// </summary>
    public static class MachineConsts
    {
        /// <summary>Positive value (= 1E-8); values which are less than this given value are shown to be 0.
        /// </summary>
        public const double Epsilon = 1E-8;

        /// <summary>Positive value (= 1E-11), which is less than <see cref="Epsilon"/> and can be used for 
        /// example as exit condition of algorithms.
        /// </summary>
        public const double TinyEpsilon = 1E-11;

        /// <summary>Positive value (= 1E-13), which is less than <see cref="Epsilon"/> and 
        /// <see cref="TinyEpsilon"/>.
        /// </summary>
        public const double SuperTinyEpsilon = 1E-13;

        /// <summary>Positive value (= 1E-16), which is almost equal to the real machine precision
        /// of a <see cref="System.Double"/>.
        /// </summary>
        public const double ExtremeTinyEpsilon = 1E-16;

        /// <summary>The smallest number such that 1.0 + <c>DBLEpsilon</c> != 1.0.
        /// </summary>
        /// <remarks>Here we set this number to the constant <c>DBL_EPSILON</c> in C++, i.e. in <c>limits.h</c> and
        /// we assume the same number.</remarks>
        public const double DBLEpsilon = 2.2204460492503131e-016;

        /// <summary>Minus the logarithm of <see cref="DBLEpsilon"/> which is the smalles number such that 1.0 + <c>DBLEpsilon</c> != 1.0.
        /// </summary>
        public const double MinusLnOfDBLEpsilon = 36.043653389117154;

        /// <summary>Positive value (= 1E+16) which represents a very large number which is also
        /// 1/<see cref="ExtremeTinyEpsilon"/>.
        /// </summary>
        public const double ExtremLargeValue = 1E+16;
    }
}