﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

namespace Dodoni.MathLibrary.NumericalIntegrators
{
    /// <summary>Serves as interface for a non-adaptive numerical integrator which supports some caching functionality, i.e.
    /// <para>
    /// \int_a^b w(x) * f(x) dx \approx \sum_{j=0}^N w(x_j) * f(x_j, j),
    /// </para>
    /// where f is the function to integrate and w is some strictly positive weight function.
    /// </summary>
    /// <remarks>If several numerical integrations are needed for a similar set of parameters one may store some pre-calculated values in
    /// one (or more) array(s) and enter these values into the function value, thus we write f(x,k) instead of f(x), where x = x_k for some
    /// at the calculation start already known array {x_0,x_1,...x_n} of abscissas.
    /// </remarks>
    public interface ICacheableOneDimNumericalIntegrator : IOneDimNumericalIntegrator
    {
        /// <summary>Gets or sets the function to integrate.
        /// </summary>
        /// <value>The function to integrate.</value>
        new CacheableOneDimNumericalIntegrator.FunctionToIntegrate FunctionToIntegrate
        {
            get;
            set;
        }

        /// <summary>Gets the factory for further <see cref="ICacheableOneDimNumericalIntegrator"/> objects of the same type and with the same configuration.
        /// </summary>
        /// <value>The factory for further <see cref="ICacheableOneDimNumericalIntegrator"/> objects of the same type and with the same configuration.</value>
        new CacheableOneDimNumericalIntegrator Factory
        {
            get;
        }
    }
}