﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.ProbabilityTheory.MonteCarloEngine
{
    /// <summary>Serves as interface for a specified Random Number Generator Library, for example Intel's Math Kernel Library (MKL), AMD's Core Math Library (ACML) etc.
    /// </summary>
    public interface IRandomNumberLibrary
    {
        /// <summary>Gets the collection of Pseudo-Random Number Generators provided by the library.
        /// </summary>
        /// <value>The collection of Pseudo-Random Number Generator.</value>
        IEnumerable<IPseudoRandomNumberGenerator> PseudoRandomNumberGenerators
        {
            get;
        }

        /// <summary>Gets the collection of Quasi-Random Number Generators provided by the library.
        /// </summary>
        /// <value>The collection of Quasi-Random Number Generator.</value>
        IEnumerable<IQuasiRandomNumberGenerator> QuasiRandomNumberGenerator
        {
            get;
        }

        /// <summary>Gets the name of the Random Number Generator Library.
        /// </summary>
        /// <value>The name of the Random Number Generator Library.</value>
        IdentifierString Name { get; }

        /// <summary>Gets a description of the Random Number Generator Library.
        /// </summary>
        /// <value>The description of the Random Number Generator Library.</value>
        string Description { get; }
    }
}