﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using Dodoni.BasicComponents;
using System.ComponentModel.Composition;

namespace Dodoni.MathLibrary.Basics.LowLevel.Native
{
    public partial class AcmlFFTNativeWrapper : FFT.ILibrary
    {
        /// <summary>Serves as factory for one-dimensional Fast-Fourier transformations with respect to AMD's ACML Library.
        /// </summary>
        internal class AcmlOneDimFourierTransformationFactory : IOneDimFourierTransformationFactory
        {
            #region public constructors

            /// <summary>Initializes a new instance of the <see cref="AcmlOneDimFourierTransformationFactory"/> class.
            /// </summary>
            public AcmlOneDimFourierTransformationFactory()
            {
            }
            #endregion

            #region public methods

            #region FFT.IOneDimFourierTransformationFactory Members

            /// <summary>Gets a specific one-dimensional Fourier transformation.
            /// </summary>
            /// <param name="length">The length, i.e. the number of complex coefficients.</param>
            /// <returns>
            /// A specific one-dimensional Fourier transformation.
            /// </returns>
            public FFT.IOneDimensional Create(int length)
            {
                return new AcmlOneDimFourierTransformation(length);
            }

            /// <summary>Gets a specific one-dimensional real Fourier transformation.
            /// </summary>
            /// <param name="length">The length, i.e. the number of real (input) coefficients.</param>
            /// <returns>
            /// A specific one-dimensional real Fourier transformation.
            /// </returns>
            public FFT.IOneDimensionalRealData CreateRealTransformation(int length)
            {
                return new AcmlOneDimRealDataFourierTransformation(length);
            }
            #endregion

            /// <summary>Returns a <see cref="System.String"/> that represents this instance.
            /// </summary>
            /// <returns>A <see cref="System.String"/> that represents this instance.
            /// </returns>
            public override string ToString()
            {
                return "ACML";
            }
            #endregion
        }
    }
}