﻿/*
   Copyright (C) 2011-2013 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using Dodoni.BasicComponents;

namespace Dodoni.MathLibrary.ProbabilityTheory.MonteCarloEngine
{
    /// <summary>Provides settings for Random Number Sequences with respect to Intel's MKL Library.
    /// </summary>
    public partial class MklRandomNumberSequence
    {
        /// <summary>Represents the generation mode provided for random numbers uniformly distributed over a specified interval.
        /// </summary>
        public class UniformGenerationMode : RandomNumberSequence.UniformGenerationMode
        {
            /// <summary>The standard approach.
            /// </summary>
            public static readonly UniformGenerationMode Standard = new UniformGenerationMode("Standard", 0, MklResources.Generation_StandardApproach);

            /// <summary>A accurate approach, perhaps slower than the <see cref="Standard"/> approach.
            /// </summary>
            public static readonly UniformGenerationMode Accurate = new UniformGenerationMode("Accurate", 1 << 30, MklResources.Generation_Accurate);

            /// <summary>The magic number, i.e. ID needed by Intel's MKL library (VSL).
            /// </summary>
            internal readonly int MagicNumber;

            /// <summary>Initializes a new instance of the <see cref="UniformGenerationMode"/> class.
            /// </summary>
            /// <param name="name">The name of the generation method for uniform distributed random numbers.</param>
            /// <param name="magicNumber">The magic number</param>
            /// <param name="annotation">The annotation (description) of the generation method.</param>
            private UniformGenerationMode(string name, int magicNumber, string annotation)
                : base(IdentifierString.Create(name), annotation)
            {
                MagicNumber = magicNumber;
            }
        }
    }
}