﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.Finance.MarketConventionTemplates
{
    /// <summary>A logging for standard market conventions, i.e. extensions for <see cref="ILogging"/> objects.
    /// </summary>
    public static class MarketConventionLogging
    {
        /// <summary>Adds default value log file messages with respect to market conventions into a specific log file.
        /// </summary>
        /// <param name="loggingObject">The logging object.</param>
        /// <param name="marketConventions">The market conventions which are taken into account.</param>
        /// <param name="userMarketConventionInput">The user input of market conventions.</param>
        public static void AddLogFileMessages(this ILogging loggingObject, ReadOnlyMoneyMarketConventions marketConventions, MoneyMarketConventions userMarketConventionInput = null)
        {
            if ((loggingObject != null) && (marketConventions != null))
            {
                if ((userMarketConventionInput == null) || (userMarketConventionInput.BusinessDayConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Business day convention", marketConventions.BusinessDayConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.BusinessDaysToSettleState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Business days to settle", marketConventions.BusinessDaysToSettle);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.DayCountConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Day count convention", marketConventions.DayCountConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.CapletTenorConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Caplet tenor convention", marketConventions.CapletTenorConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FutureBasePointValueState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Future base point value", marketConventions.FutureBasePointValue);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.LiborRateRoundingRuleState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Libor Rounding Rule", marketConventions.LiborRateRoundingRule.Name);
                }
            }
        }

        /// <summary>Adds default value log file messages with respect to market conventions into a specific log file.
        /// </summary>
        /// <param name="loggingObject">The logging object.</param>
        /// <param name="marketConventions">The market conventions which are taken into account.</param>
        /// <param name="userMarketConventionInput">The user input of market conventions.</param>
        public static void AddLogFileMessages(this ILogging loggingObject, ReadOnlySwapMarketConventions marketConventions, SwapMarketConventions userMarketConventionInput = null)
        {
            if ((loggingObject != null) && (marketConventions != null))
            {
                if ((userMarketConventionInput == null) || (userMarketConventionInput.BusinessDaysToSettleState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Business days to settle", marketConventions.BusinessDaysToSettle);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FixedBusinessDayConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Fixed Business day convention", marketConventions.FixedBusinessDayConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FixedDayCountConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Fixed Day Count convention", marketConventions.FixedDayCountConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FixedFrequencyState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Fixed frequency", marketConventions.FixedFrequency.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FloatingBusinessDayConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Floating Business day convention", marketConventions.FloatingBusinessDayConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FloatingDayCountConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Floating Day Count convention", marketConventions.FloatingDayCountConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.FloatingFrequencyState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Floating frequency", marketConventions.FloatingFrequency.Name);
                }
            }
        }

        /// <summary>Adds default value log file messages with respect to market conventions into a specific log file.
        /// </summary>
        /// <param name="loggingObject">The logging object.</param>
        /// <param name="marketConventions">The market conventions which are taken into account.</param>
        /// <param name="userMarketConventionInput">The user input of market conventions.</param>
        public static void AddLogFileMessages(this ILogging loggingObject, ReadOnlyBondMarketConventions marketConventions, BondMarketConventions userMarketConventionInput = null)
        {
            if ((loggingObject != null) && (marketConventions != null))
            {
                if ((userMarketConventionInput == null) || (userMarketConventionInput.BusinessDayConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Business day convention", marketConventions.BusinessDayConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.BusinessDaysToSettleState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Business days to settle", marketConventions.BusinessDaysToSettle);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.DayCountConventionState != ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Day count convention", marketConventions.DayCountConvention.Name);
                }
                if ((userMarketConventionInput == null) || (userMarketConventionInput.CouponFrequencyState!= ConventionState.UserInput))
                {
                    loggingObject.LogFile.AddDefaultValueMessage("Coupon frequency", marketConventions.CouponFrequency.Name);
                }
            }
        }
    }
}