﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.Finance.StandardModels
{
    /// <summary>Provides extension methods concerning standard option models, i.e. for Black, Bachelier etc.
    /// </summary>
    public static class StandardModelExtensions
    {
        #region nested classes

        private class Wrapper : IEuropeanOptionPricer
        {
            #region private members

            private IConstantVolatilityStandardEuropeanOption m_StandardOption;
            private double m_Volatility;
            #endregion


            internal Wrapper(IConstantVolatilityStandardEuropeanOption standardOption, double volatility)
            {
                m_StandardOption = standardOption;
                m_Volatility = volatility;
            }
            #region IEuropeanOptionPricer Members

            public double TimeToExpiration
            {
                get { return m_StandardOption.TimeToExpiration; }
                set { m_StandardOption.TimeToExpiration = value; }
            }

            public double Forward
            {
                get { return m_StandardOption.Forward; }
                set { m_StandardOption.Forward = value; }
            }

            public void ReInitialize()
            {

            }

            public double GetUndiscountedValue(double strike)
            {
                m_StandardOption.Strike = strike;
                return m_StandardOption.GetUndiscountedValue(m_Volatility);
            }

            #endregion

            #region IOperable Members

            public bool IsOperable
            {
                get { return true; }
            }
            #endregion
        }
        #endregion

        #region public (static) methods

        /// <summary>Returns a <see cref="IEuropeanOptionPricer"/> object that encapsulate the current instance.
        /// </summary>
        /// <param name="constantVolatilityStandardEuropeanOption">A specified European option based on a model with a constant volatility (i.e. Black, Bachelier etc.).</param>
        /// <param name="volatility">The volatility parameter.</param>
        /// <returns></returns>
        public static IEuropeanOptionPricer AsEuropeanOptionPricer(this IConstantVolatilityStandardEuropeanOption constantVolatilityStandardEuropeanOption, double volatility)
        {
            return new Wrapper(constantVolatilityStandardEuropeanOption, volatility);
        }
        #endregion
    }
}