﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.Finance;
using Dodoni.BasicComponents.Utilities;

namespace Dodoni.Finance.CommonMarketUsages.HolidayCalendars.America
{
    /// <summary>Represents the public holiday calendar for the United States (http://www.opm.gov/fedhol/), 'US'.
    /// </summary>
    [CustomHolidayCalendar("US", HolidayCalendarRegion.America, "Dodoni.Finance.CommonMarketUsages.HolidayCalendars.HolidayNameResources", Description = "_US")]
    internal enum UnitedStates
    {
        /// <summary>'New year' public holiday, represents the first January of each year. 
        /// The holiday rolling type is set to <see cref="HolidayRollingType.ForwardRolling"/>.
        /// </summary>
        [FixHoliday("NewYear", 1, Month.January, HolidayRollingType = HolidayRollingType.ForwardRolling)]
        NewYearsDay,

        /// <summary>Martin Luther King's birthday (third Monday in January). 
        /// </summary>
        [MonthWeekdayHoliday("MartinLutherKingsBirthDay", DayOfWeek.Monday, Month.January, MonthWeekdayHoliday.WeekIndex.Third)]
        MartinLutherDay,

        /// <summary>Washington's birthday (third Monday in February)
        /// </summary>
        [MonthWeekdayHoliday("WashingtonsBirthDay", DayOfWeek.Monday, Month.February, MonthWeekdayHoliday.WeekIndex.Third)]
        Washington,

        /// <summary>Memorial Day (last Monday in May).
        /// </summary>
        [MonthWeekdayHoliday("MemorialDay", DayOfWeek.Monday, Month.May, MonthWeekdayHoliday.WeekIndex.Last)]
        MemorialDay,

        /// <summary>Independence Day, July 4th (moved to Monday if Sunday or Friday if Saturday.
        /// </summary>
        [FixHoliday("USIndependenceDay", 4, Month.July, HolidayRollingType = HolidayRollingType.SundayToMondaySaturdayToFriday)]
        IndependenceDay,

        /// <summary>The 'labour day', i.e. 1st of may, the holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.
        /// </summary>
        [FixHoliday("USLabourDay", 1, Month.May, HolidayRollingType = HolidayRollingType.NoRolling)]
        LabourDay,

        /// <summary>Columbus Day, second Monday in October.
        /// </summary>
        [MonthWeekdayHoliday("USColumbusDay", DayOfWeek.Monday, Month.October, MonthWeekdayHoliday.WeekIndex.Second)]
        ColumbusDay,

        /// <summary>Veterans' Day, November 11th (moved to Monday if Sunday or Friday if Saturday).
        /// </summary>
        [FixHoliday("USVeteransDay", 11, Month.November, HolidayRollingType = HolidayRollingType.SundayToMondaySaturdayToFriday)]
        VeteransDay,

        /// <summary>Thanksgiving Day, i.e. fourth Thursday in November.
        /// </summary>
        [MonthWeekdayHoliday("ThanksgivingDay", DayOfWeek.Thursday, Month.November, MonthWeekdayHoliday.WeekIndex.Fourth)]
        ThanksgivingDay,

        /// <summary>The 'christmas day', i.e. 25st december. The holiday rolling type is set to
        /// <see cref="HolidayRollingType.ForwardRolling"/>.
        /// </summary>
        [FixHoliday("ChristmasDay", 25, Month.December, HolidayRollingType = HolidayRollingType.ForwardRolling)]
        ChristmasDay
    }
}