﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;

using Dodoni.Finance;
using Dodoni.BasicComponents.Utilities;

namespace Dodoni.Finance.CommonMarketUsages.HolidayCalendars.Europe
{
    /// <summary>Represents the list of holidays taken into account for the holiday calendar 'FFM:Xetra' (http://deutsche-boerse.com/), i.e.
    /// Frankfurt/Main, Germany.
    /// </summary>
    [CustomHolidayCalendar("FFM:Xetra", HolidayCalendarRegion.Europe, "Dodoni.Finance.CommonMarketUsages.HolidayCalendars.HolidayNameResources", Description = "_FFM_Xetra")]
    internal enum FFM_Xetra
    {
        /// <summary>'New year' public holiday.
        /// </summary>
        /// <remarks>Represents the first January of each year. The holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [FixHoliday("NewYear", 1, Month.January, HolidayRollingType = HolidayRollingType.NoRolling)]
        NewYearsDay,

        /// <summary>The 'good friday'.
        /// </summary>
        /// <remarks>Represents good friday, the holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [EasterDependingHoliday("GoodFriday", -3, HolidayRollingType = HolidayRollingType.NoRolling)]
        GoodFriday,

        /// <summary>The 'easter monday'.
        /// </summary>
        /// <remarks>Represents easter monday, the holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [EasterDependingHoliday("EasterMonday", 0, HolidayRollingType = HolidayRollingType.NoRolling)]
        EasterMonday,

        /// <summary>The 'labour day'.
        /// </summary>
        /// <remarks>Represents the Laour day, i.e. 1st of may. The holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [FixHoliday("LabourDay", 1, Month.May, HolidayRollingType = HolidayRollingType.NoRolling)]
        LabourDay,

        /// <summary>The 'christmas eve'.
        /// </summary>
        /// <remarks>Represents christmas eve, i.e. 24st december. The holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [FixHoliday("ChristmasEve", 24, Month.December, HolidayRollingType = HolidayRollingType.NoRolling)]
        ChristmasEve,

        /// <summary>The 'christmas day'.
        /// </summary>
        /// <remarks>Represents christmas day, i.e. 25st december. The holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [FixHoliday("ChristmasDay", 25, Month.December, HolidayRollingType = HolidayRollingType.NoRolling)]
        ChristmasDay,

        /// <summary>The 'boxing day'.
        /// </summary>
        /// <remarks>Represents boxing date, i.e. 26st december. The holiday rolling type is set to
        /// <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [FixHoliday("BoxingDay", 26, Month.December, HolidayRollingType = HolidayRollingType.NoRolling)]
        BoxingDay,

        /// <summary>The 'new year eve'.
        /// </summary>
        /// <remarks>Represents new years eve, i.e. 31st december. The holiday rolling type is set to <see cref="HolidayRollingType.NoRolling"/>.</remarks>
        [FixHoliday("NewYearsEve", 31, Month.December, HolidayRollingType = HolidayRollingType.NoRolling)]
        NewYearsEve
    }
}