﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using NUnit.Framework;

namespace Dodoni.Finance.CommonMarketUsages.DayCountConventions
{
    /// <summary>Serves as class for unit tests with respect to <see cref="ActualActualAFB"/>.
    /// </summary>
    [TestFixture]
    public class ActualActualAFB_Tests
    {
        /// <summary>Serves as unit test for <see cref="IDayCountConvention.GetYearFraction(DateTime, DateTime, DateTime?, DateTime?)"/>.
        /// </summary>
        /// <param name="startDate">The start date of the period.</param>
        /// <param name="endDate">The end date of the period.</param>
        /// <param name="expectedYearFraction">The expected year fraction.</param>
        [TestCaseSource("GetYearFractionTestCaseData")]
        public void GetYearFraction_TestCase(DateTime startDate, DateTime endDate, double expectedYearFraction)
        {
            double actualYearFraction = DayCountConvention.ActualActual.AFB.GetYearFraction(startDate, endDate);

            Assert.That(actualYearFraction, Is.EqualTo(expectedYearFraction).Within(1E-6));
        }

        /// <summary>Gets the test case data for <see cref="GetYearFraction_TestCase(DateTime, DateTime, double)"/>.
        /// </summary>
        /// <value>The test case data for <see cref="GetYearFraction_TestCase(DateTime, DateTime, double)"/>.</value>
        public IEnumerable<TestCaseData> GetYearFractionTestCaseData
        {
            get
            {
                /* The following unit tests are inspired by the unit tests of the QuantLib project; 
                 * Copyright (C) 2003 RiskMap srl
                 * Copyright (C) 2006 Piter Dias
                 */
                yield return new TestCaseData(new DateTime(2003, 11, 1), new DateTime(2004, 5, 1), 0.497267759563);
                yield return new TestCaseData(new DateTime(1999, 2, 1), new DateTime(1999, 7, 1), 0.410958904110);
                yield return new TestCaseData(new DateTime(1999, 7, 1), new DateTime(2000, 7, 1), 1.0);
                yield return new TestCaseData(new DateTime(2002, 8, 15), new DateTime(2003, 7, 15), 0.915068493151);
                yield return new TestCaseData(new DateTime(2003, 7, 15), new DateTime(2004, 1, 15), 0.504109589041);
                yield return new TestCaseData(new DateTime(1999, 7, 30), new DateTime(2000, 1, 30), 0.504109589041);
                yield return new TestCaseData(new DateTime(2000, 1, 30), new DateTime(2000, 6, 30), 0.41530054644);

                // use QuantLib 1.2.0:
                yield return new TestCaseData(new DateTime(2003, 11, 1), new DateTime(2004, 5, 1), 0.497267759562842);
                yield return new TestCaseData(new DateTime(2001, 10, 1), new DateTime(2004, 5, 1), 2.580821917808220);
                yield return new TestCaseData(new DateTime(2000, 2, 26), new DateTime(2002, 8, 15), 2.467213114754100);
                yield return new TestCaseData(new DateTime(2000, 2, 29), new DateTime(2003, 2, 18), 2.969945355191260);
                yield return new TestCaseData(new DateTime(2003, 5, 18), new DateTime(2012, 6, 3), 9.043835616438360);
            }
        }
    }
}