﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Configuration;
using System.Windows.Forms;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.XLBasicComponents.LogFile;
using Dodoni.XLBasicComponents.Utilities;

namespace Dodoni.XLBasicComponents
{
    /// <summary>Represents the windows form for the configuration of Dodoni.NET. Moreover this class
    /// managed the configuration file for the Excel interface of Dodoni.NET in general.
    /// </summary>
    internal partial class ConfigurationForm : Form
    {
        #region private members

        /// <summary>A mapping between the user control and the <see cref="TreeNode"/> representation.
        /// </summary>
        private Dictionary<TreeNode, IConfigurationUserControl> m_Controls;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="ConfigurationForm"/> class.
        /// </summary>
        public ConfigurationForm()
        {
            InitializeComponent();

            m_Controls = new Dictionary<TreeNode, IConfigurationUserControl>();

            TreeNode root = new TreeNode("General");
            root.Name = "General";
            m_Controls.Add(root, new UserControlConfigurationGeneral());
            treeViewMain.Nodes.Add(root);

            TreeNode loggingRoot = new TreeNode("Logging");
            loggingRoot.Name = "Logging";
            m_Controls.Add(loggingRoot, new UserControlConfigurationLogFile());
            treeViewMain.Nodes.Add(loggingRoot);

            /* load external, additional configuration user controls: */
            ExcelAddIn.Configuration.RaiseConfigurationUserControlRegister(treeViewMain, new ExcelConfiguration.ConfigurationUserControlRegisterEventArgs(m_Controls));
        }
        #endregion

        #region private methods

        /// <summary>Handles the AfterSelect event of the treeViewMain control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.TreeViewEventArgs"/> instance containing the event data.</param>
        private void treeViewMain_AfterSelect(object sender, TreeViewEventArgs e)
        {
            IConfigurationUserControl userControl;
            if (m_Controls.TryGetValue(e.Node, out userControl) == true)
            {
                mainGroupBox.Hide();

                mainGroupBox.Controls.Clear();
                UserControl control = (UserControl)userControl;
                control.Left = 10;
                control.Top = 15;
                control.Height = mainGroupBox.Height - 25;
                control.Width = mainGroupBox.Width - 15;
                control.Anchor = AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top;
                mainGroupBox.Text = userControl.Title;

                mainGroupBox.Controls.Add(control);
                mainGroupBox.Show();
            }
        }

        /// <summary>Handles the Click event of the OK button.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void bOK_Click(object sender, EventArgs e)
        {
            bool configurationChanged = false;
            foreach (IConfigurationUserControl userControl in m_Controls.Values)
            {
                if (userControl.ConfigurationChanged == true)
                {
                    configurationChanged = true;
                    userControl.StoreConfiguration();
                }
            }
            if (configurationChanged == true)
            {
                MessageBox.Show("Write configuration file. Perhaps a restart necessary.", "Dodoni.NET Configuration", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            Close();
        }
        #endregion
    }
}
