﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.IO;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;

namespace Dodoni.XLBasicComponents.IO
{
    /// <summary>Serves as interface for writing objects into a specific stream.
    /// </summary>
    public interface IObjectStreamWriter : IOperable
    {
        /// <summary>Opens the <see cref="IObjectStreamWriter"/> object with a specific <see cref="StreamWriter"/> object.
        /// </summary>
        /// <param name="streamWriter">The stream writer.</param>
        /// <remarks>Except for the first time call <see cref="IObjectStreamWriter.Close(bool)"/> before using a other <see cref="StreamWriter"/> object.</remarks>
        void Open(StreamWriter streamWriter);

        /// <summary>Closes the <see cref="IObjectStreamWriter"/> object and the underlying stream,
        /// and releases any system resources associated with the reader.
        /// </summary>
        /// <param name="closeEmbeddedStream">A value indicating whether the underlying <see cref="StreamWriter"/> should be closed as well.</param>
        void Close(bool closeEmbeddedStream = true);

        /// <summary>Writes the <see cref="GuidedExcelDataQuery"/> representation of a specific <see cref="ExcelPoolItem"/> object into the stream.
        /// </summary>
        /// <param name="value">The <see cref="ExcelPoolItem"/> to store.</param>
        /// <remarks>The stream must be filled in a valid order taken into account the dependency structure, i.e. first 'independent' objects will be stored, afterwards
        /// objects which needes as input 'independent' objects will be stored etc. However implementations stores the <see cref="GuidedExcelDataQuery"/> representation 
        /// of <paramref name="value"/> only, thus the dependency structure, must be managed by the caller of the method.
        /// </remarks>
        void WriteObject(ExcelPoolItem value);
    }
}