﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.XLBasicComponents
{
    /// <summary>Extension methods (properties) of the logfile factory.
    /// </summary>
    public static class ExcelLogFileMessageTypeExtensions
    {
        #region internal static (readonly) members

        /// <summary>The info logfile message type which indicates that a specific Excel cell has been ignored because it is empty.
        /// </summary>
        internal static readonly InfoLogFileMessageType sm_EmptyExcelCellIgnore = new InfoLogFileMessageType("Ignored empty excel cell", new Guid(0x9f228aec, 0x94d7, 0x47a5, 0x91, 0xad, 0xd8, 0xf5, 0xbc, 0xab, 0xe2, 0x27));

        /// <summary>The info logfile message type which indicates that the Excel cell data validation fails, i.e. it was not possible to add a 
        /// dropdown list to a specific Excel cell.
        /// </summary>
        internal static readonly InfoLogFileMessageType sm_ExcelCellDropdownListCreationFails = new InfoLogFileMessageType("Excel cell validation failure", new Guid(0x1bd41881, 0xb9eb, 0x49bc, 0x9b, 0xf0, 0x94, 0x4e, 0x7a, 0xcd, 0x36, 0x6c));

        /// <summary>The fatal error message type which indicates that the size of the Excel range output can not be calculated, i.e. a <see cref="ExcelDna.Integration.XlCallException"/> has been thrown.
        /// </summary>
        internal static readonly FatalErrorLogFileMessageType sm_ExcelRangeSizeCalculationFails = new FatalErrorLogFileMessageType("Excel range size problem", new Guid(0xbcaf5857, 0xcb2c, 0x4e99, 0xb4, 0xd2, 0x25, 0xa2, 0x94, 0xb6, 0x7e, 0xc0));
        #endregion

        #region public (static) methods

        /// <summary>The info logfile message type which indicates that a specific Excel cell has been ignored because is it empty.
        /// </summary>
        /// <param name="infoMessagePool">The info message pool.</param>
        /// <returns>The <see cref="InfoLogFileMessageType"/> which is use to indicates that a specific Excel cell has been ignored because it is empty.</returns>
        public static InfoLogFileMessageType EmptyExcelCellIgnored(this InfoMessageTypePool infoMessagePool)
        {
            return sm_EmptyExcelCellIgnore;
        }

        /// <summary>The info logfile message type which indicates that the Excel cell data validation fails, i.e. it was not possible to add a 
        /// dropdown list to a specific Excel cell.
        /// </summary>
        /// <param name="infoMessagePool">The info message pool.</param>
        /// <returns>The <see cref="InfoLogFileMessageType"/> which is use to indicates that the Excel cell data validation fails, it was not possible to add a 
        /// dropdown list to a specific Excel cell.</returns>
        public static InfoLogFileMessageType ExcelCellDropDownListCreationFails(this InfoMessageTypePool infoMessagePool)
        {
            return sm_ExcelCellDropdownListCreationFails;
        }

        /// <summary>The fatal error logfile message type which indicates that the size of the Excel range output can not be calculated, i.e. a <see cref="ExcelDna.Integration.XlCallException"/> has been thrown.
        /// </summary>
        /// <param name="fatalErrorMessagePool">The fatal error message pool.</param>
        /// <returns>The <see cref="FatalErrorLogFileMessageType"/> object which is used to indicate that the size of the Excel range output can not be calculated, 
        /// i.e. a <see cref="ExcelDna.Integration.XlCallException"/> has been thrown.</returns>
        public static FatalErrorLogFileMessageType ExcelRangeSizeCalculationFails(this FatalErrorMessageTypePool fatalErrorMessagePool)
        {
            return sm_ExcelRangeSizeCalculationFails;
        }
        #endregion
    }
}