﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents.LogFileFactory;
using Dodoni.BasicComponents.LogFileFactory.LogFileMsgTypes;

namespace Dodoni.XLBasicComponents.LogFile
{
    /// <summary>Represents a row of the global log file.
    /// </summary>
    internal class GlobalLogFileRow : LogFileRow
    {
        #region private members

        /// <summary>The object type ID, i.e. the unique ID of the object type (for example Holiday calendar pool etc.).
        /// </summary>
        private Guid m_ObjectTypeID;

        /// <summary>The name of the object type, for example Holiday calendar pool etc.
        /// </summary>
        private string m_ObjectTypeName;

        /// <summary>The object ID, i.e. the unique ID of the specific object.
        /// </summary>
        private Guid m_ObjectID;

        /// <summary>The name of the object.
        /// </summary>
        private string m_ObjectName;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="GlobalLogFileRow"/> class.
        /// </summary>
        /// <param name="messageType">The message type.</param>
        /// <param name="message">The message.</param>
        /// <param name="objectTypeID">The object type ID.</param>
        /// <param name="objectTypeName">The name of the object type.</param>
        /// <param name="objectID">The ID of the specific object.</param>
        /// <param name="objectName">The name of the specific object.</param>
        /// <exception cref="ArgumentNullException">Thrown, if the <paramref name="messageType"/> is <c>null</c>.</exception>
        public GlobalLogFileRow(ILogFileMessageType messageType, string message, Guid objectTypeID, string objectTypeName, Guid objectID, string objectName)
            : base(messageType, message)
        {
            m_ObjectTypeID = objectTypeID;
            m_ObjectTypeName = (objectTypeName != null) ? objectTypeName : String.Empty;
            m_ObjectID = objectID;
            m_ObjectName = (objectName != null) ? objectName : String.Empty;
        }
        #endregion

        #region public properties

        /// <summary>Gets the object type ID.
        /// </summary>
        /// <value>The object type ID.</value>
        public Guid ObjectTypeID
        {
            get { return m_ObjectTypeID; }
        }

        /// <summary>Gets the name of the object type, for example Holiday Calendar Pool etc.
        /// </summary>
        /// <value>The name of the object type.</value>
        public string ObjectTypeName
        {
            get { return m_ObjectTypeName; }
        }

        /// <summary>Gets the object ID.
        /// </summary>
        /// <value>The object ID.</value>
        public Guid ObjectID
        {
            get { return m_ObjectID; }
        }

        /// <summary>Gets the name of the object.
        /// </summary>
        /// <value>The name of the object.</value>
        public string ObjectName
        {
            get { return m_ObjectName; }
        }
        #endregion
    }
}