﻿using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents.Containers;
using Dodoni.BasicComponents.LogFileFactory;

namespace Dodoni.XLGuidedTour
{
    public class MyFirstClass : IInfoOutputQueriable, ILogging
    {
        #region private static members

        /// <summary>The unique object type ID for the log file.
        /// </summary>
        private static Guid sm_ObjectTypeID = new Guid(0x689b08d0, 0xfb0a, 0x4f84, 0x8c, 0xaa, 0xec, 0x45, 0xf0, 0x17, 0x5e, 0xa5);
        #endregion

        #region private members

        /// <summary>The factor.
        /// </summary>
        private double m_Factor;

        /// <summary>The log file.
        /// </summary>
        private ILogFile m_LogFile;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="MyFirstClass"/> class.
        /// </summary>
        /// <param name="factor">The factor.</param>
        public MyFirstClass(double factor)
        {
            m_Factor = factor;
            m_LogFile = LoggingFactory.CreateLogFile("myFirstClass", sm_ObjectTypeID);
            
            // a simple example:
            m_LogFile.AddMessage(LoggingFactory.Infos.CreateObject, "factor: " + factor.ToString());
        }
        #endregion

        #region ILogging Members

        /// <summary>Gets the <see cref="ILogFile"/> object.
        /// </summary>
        /// <value>The <see cref="ILogFile"/> object.</value>
        public ILogFile LogFile
        {
            get { return m_LogFile; }
        }
        #endregion

        #region IInfoOutputQueriable Members

        /// <summary>Gets the info-output level of detail.
        /// </summary>
        /// <value>The info-output level of detail.</value>
        public InfoOutputDetailLevel InfoOutputDetailLevel
        {
            get { return InfoOutputDetailLevel.Full; }
        }

        /// <summary>Sets the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel"/> property.
        /// </summary>
        /// <param name="infoOutputDetailLevel">The info-output level of detail.</param>
        /// <returns>
        /// A value indicating whether the <see cref="IInfoOutputQueriable.InfoOutputDetailLevel"/> has been set to <paramref name="infoOutputDetailLevel"/>.
        /// </returns>
        public bool TrySetInfoOutputDetailLevel(InfoOutputDetailLevel infoOutputDetailLevel)
        {
            return (infoOutputDetailLevel == InfoOutputDetailLevel.Full);
        }

        /// <summary>Gets informations of the current object as a specific <see cref="InfoOutput"/> instance.
        /// </summary>
        /// <param name="infoOutput">The <see cref="InfoOutput"/> object which is to be filled with informations concering the current instance.</param>
        /// <param name="categoryName">The name of the category, i.e. all informations will be added to these category.</param>
        public void FillInfoOutput(InfoOutput infoOutput, string categoryName = "General")
        {
            InfoOutputPackage infoOutputPackage = infoOutput.AcquirePackage(categoryName);
           infoOutputPackage.Add("Factor", m_Factor);
            // you can add DataTable objects as well
        }
        #endregion
    }
}