﻿using System;
using System.Text;
using System.Collections.Generic;

using ExcelDna.Integration;
using Dodoni.XLBasicComponents;

using Dodoni.BasicComponents;
using Dodoni.BasicComponents.Containers;

namespace Dodoni.XLGuidedTour
{
    public static class ToyPoolItemCreationUDFs
    {
        /// <summary>Represents the <see cref="ExcelPoolItemType"/> of 'myFirstObjectType',
        /// {6394B82E-0092-4C9D-AD4C-D7CA00DC694E}.
        /// </summary>
        private static readonly ExcelPoolItemType sm_ExcelPoolItemType = new ExcelPoolItemType("MyFirstObjectType", new Guid(0x6394b82e, 0x92, 0x4c9d, 0xad, 0x4c, 0xd7, 0xca, 0x0, 0xdc, 0x69, 0x4e));

        [ExcelFunction(Name = "myCreateMyFirstObject")]
        public static object CreateMyFirstExcelPoolItem(
            [ExcelArgument(Name = "generalPropertyNameRange", Description = "A single row or column Range which contains the names of general properties", AllowReference = true)]
            object xlGeneralPropertyNameRange,
            [ExcelArgument(Name = "generalPropertyValueRange", Description = "A single row or column Range which contains the values of general properties", AllowReference = true)]            
            object xlGeneralPropertyValueRange)
        {
            try
            {
                IExcelDataQuery generalPropertyDataQuery = ExcelDataQuery.Create(xlGeneralPropertyNameRange, xlGeneralPropertyValueRange);
                string objectName = generalPropertyDataQuery.GetObjectName();
                double factor = generalPropertyDataQuery.GetRequiredPropertyValue<double>("Factor");
                generalPropertyDataQuery.QueryCompleted();

                IInfoOutputQueriable value = null;  // dummy value -> NullArgumentException
                value = new MyFirstClass(factor);

                ExcelPoolItem excelPoolItem = new ExcelPoolItem(value, objectName, sm_ExcelPoolItemType, generalPropertyDataQuery);
                ExcelPool.InsertObject(excelPoolItem);

                return objectName.ToTimeStampString();
            }
            catch (Exception e)
            {
                return ExcelDataConverter.GetExcelRangeErrorMessage(e);
            }
        }
    }
}