﻿using System;
using System.Text;

using ExcelDna.Integration;
using Dodoni.XLBasicComponents;

using Dodoni.BasicComponents.Containers;

namespace Dodoni.XLGuidedTour
{
    public class WorkingPoolItemCreationUDFs
    {
        /// <summary>Represents the <see cref="ExcelPoolItemType"/> of 'myFirstObjectType',
        /// {6394B82E-0092-4C9D-AD4C-D7CA00DC694E}.
        /// </summary>
        private static readonly ExcelPoolItemType sm_ExcelPoolItemType = new ExcelPoolItemType("MyFirstObjectType2", new Guid(0x6394b82e, 0x92, 0x4c9d, 0xad, 0x4c, 0xd7, 0xca, 0x0, 0xdc, 0x69, 0x4e));

        [ExcelFunction(Name = "myCreateMyFirstObject2")]
        public static object CreateMyFirstExcelPoolItem(
            [ExcelArgument(Name = "generalPropertyNameRange", Description = "A single row or column Range which contains the names of general properties", AllowReference = true)]
            object xlGeneralPropertyNameRange,
            [ExcelArgument(Name = "generalPropertyValueRange", Description = "A single row or column Range which contains the values of general properties", AllowReference = true)]            
            object xlGeneralPropertyValueRange)
        {
            if (ExcelDnaUtil.IsInFunctionWizard())
            {
                return String.Empty;
            }
            return ExcelPool.InsertObject(TryCreateFirstObject, IdentifierNameableDictionary.Create(ExcelDataQuery.Create(xlGeneralPropertyNameRange, xlGeneralPropertyValueRange)));
        }

        /// <summary>A delegate for the <see cref="ExcelPoolItemCreator.Initialize"/> event to establish a specific <see cref="ExcelPoolItem"/> object.
        /// </summary>
        /// <param name="excelPoolInitializeEventArgs">The <see cref="Dodoni.XLBasicComponents.ExcelPoolItemCreator.InitializeEventArgs"/> instance containing the event data.</param>
        internal static void ExcelPoolItemCreator_Initialize(ExcelPoolItemCreator.InitializeEventArgs excelPoolInitializeEventArgs)
        {
            excelPoolInitializeEventArgs.Add(new ExcelPoolItemCreator(sm_ExcelPoolItemType, TryCreateFirstObject));
        }

        /// <summary>Creates a specific <see cref="DateSchedule"/> wrapped in a <see cref="ExcelPoolItem"/> object.
        /// </summary>
        /// <param name="excelDataQueries">The data needed to construct a new object.</param>
        /// <param name="value">The output object (output).</param>
        /// <param name="errorMessage">The error message or undefined if no error occurs (output).</param>
        /// <returns>A value indicating whether <paramref name="value"/> contains valid data; otherwise <paramref name="errorMessage"/> contains an error message.</returns>
        private static bool TryCreateFirstObject(IIdentifierStringDictionary<IExcelDataQuery> excelDataQueries, out ExcelPoolItem value, out string errorMessage)
        {
            try
            {
                IExcelDataQuery generalPropertyDataQuery = excelDataQueries.GetGeneralProperties();
                string objectName = generalPropertyDataQuery.GetObjectName();
                double factor = generalPropertyDataQuery.GetRequiredPropertyValue<double>("Factor");

                IInfoOutputQueriable itemValue = new MyFirstClass(factor);

                value = new ExcelPoolItem(itemValue, objectName, sm_ExcelPoolItemType, generalPropertyDataQuery);
                errorMessage = String.Empty;
                return true;
            }
            catch (Exception e)
            {
                value = null;
                errorMessage = e.Message;
                return false;
            }
        }
    }
}