﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Windows.Forms;

using ExcelDna.Integration;
using Dodoni.XLBasicComponents;

using Dodoni.XLIntegration.Utilities;
using Dodoni.XLBasicComponents.Utilities;

namespace Dodoni.XLIntegration
{
    /// <summary>Represents the Excel-AddIn for Gnuplot.
    /// </summary>
    public class GnuPlotExcelAddIn : IExcelAddIn
    {
        #region internal const members

        /// <summary>The name of the Excel menue, i.e. 'Dodoni'.
        /// </summary>
        internal const string MenuName = "Dodoni";

        /// <summary>The name of the Excel-category for utility functions.
        /// </summary>
        internal const string UtilityCategoryName = "Dodoni.Utilities";
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="GnuPlotExcelAddIn"/> class.
        /// </summary>
        public GnuPlotExcelAddIn()
        {
        }
        #endregion

        #region public methods

        #region IExcelAddIn Members

        /// <summary>Initialize the <see cref="IExcelAddIn"/>.
        /// </summary>
        public void AutoOpen()
        {
            try
            {
                ExcelAddIn.Configuration.UserControlRegister += ExcelConfiguration_UserControlRegister;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "GnuPlot Excel-AddIn: Fatal error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>Performs operations if Excel will be closed.
        /// </summary>
        public void AutoClose()
        {
        }
        #endregion

        #endregion

        #region private methods

        /// <summary>Register user controls for the configuration form, i.e. for gnuplot configuration etc.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="eventArgs">The <see cref="ExcelConfiguration.ConfigurationUserControlRegisterEventArgs"/> instance containing the event data.</param>
        private void ExcelConfiguration_UserControlRegister(TreeView sender, ExcelConfiguration.ConfigurationUserControlRegisterEventArgs eventArgs)
        {
            TreeNode graphicRoot = new TreeNode("Gnuplot");
            graphicRoot.Name = "Gnuplot";
            eventArgs.Add(graphicRoot, new GnuplotConfigUserControl());
            sender.Nodes.Add(graphicRoot);
        }
        #endregion
    }
}