﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.IO;
using System.Text;
using System.Collections.Generic;

namespace Dodoni.XLIntegration.Utilities
{
    /// <summary>Serves as <see cref="GnuplotApplication"/> implementation, where a scriptfile is the output only, i.e.
    /// one has to call gnuplot with respect to this outcome afterwards.
    /// </summary>
    internal class GnuplotApplicationScript : GnuplotApplication
    {
        #region internal constructors

        /// <summary>Initializes a new instance of the <see cref="GnuplotApplicationScript"/> class.
        /// </summary>
        /// <param name="scriptFileName">Name of the script file.</param>
        internal GnuplotApplicationScript(string scriptFileName)
        {
            m_InputStream = File.CreateText(scriptFileName);
            m_InputStream.AutoFlush = true;
        }
        #endregion

        #region public methods

        /// <summary>Closes the current instance.
        /// </summary>
        /// <param name="errorMessage">The error message, perhaps <c>null</c> (output).</param>
        /// <returns>If <c>false</c> some error occurs and <paramref name="errorMessage"/> contains a error message; otherwise
        /// <paramref name="errorMessage"/> is <c>null</c>.
        /// </returns>
        public override bool Close(out string errorMessage)
        {
            if (m_GnuplotData.Count > 0)
            {
                foreach (Gnuplot2dData data in m_GnuplotData)
                {
                    m_PlotCommandStrings.AppendLine(data.GetDataAsString());
                }
                m_InputStream.WriteLine("plot " + m_PlotCommandStrings.ToString());
            }
            m_InputStream.Close();
            m_GnuplotData.Clear();
            m_PlotCommandStrings.Clear();

            errorMessage = String.Empty;
            return true; // we can not check the syntax of the code
        }
        #endregion
    }
}