﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Windows.Forms;

using Dodoni.XLBasicComponents;

namespace Dodoni.XLIntegration.Utilities
{
    /// <summary>Represents the configuration of the export of graphics with respect to Gnuplot.
    /// </summary>
    internal partial class GnuplotConfigUserControl : UserControl, IConfigurationUserControl
    {
        #region private members

        /// <summary>The file dialog.
        /// </summary>
        private OpenFileDialog m_OpenFileDialog;

        /// <summary>A value indicating whether the user has changed the configuration.
        /// </summary>
        private bool m_ConfigurationChanged = false;
        #endregion

        #region public constructors

        /// <summary>Initializes a new instance of the <see cref="GnuplotConfigUserControl"/> class.
        /// </summary>
        public GnuplotConfigUserControl()
        {
            InitializeComponent();

            m_OpenFileDialog = new OpenFileDialog();
            m_OpenFileDialog.CheckFileExists = true;
            m_OpenFileDialog.CheckPathExists = true;
            m_OpenFileDialog.AddExtension = true;
            m_OpenFileDialog.Multiselect = false;
            m_OpenFileDialog.Title = "External graphic export program (gnuplot.exe)";
            m_OpenFileDialog.InitialDirectory = GnuplotApplication.StandardGnuPlotPath;
            m_OpenFileDialog.Filter = "GnuPlot (gnuplot.exe)|gnuplot.exe";

            RestoreConfiguration();
        }
        #endregion

        #region IConfigurationUserControl Members

        /// <summary>Gets the title of the user control in its <see cref="System.String"/> representation.
        /// </summary>
        /// <value>The title of the user control.</value>
        public string Title
        {
            get { return "Gnuplot"; }
        }

        /// <summary>Gets a value indicating whether the user changed the configuration.
        /// </summary>
        /// <value><c>true</c> if the user changed the configuration; otherwise, <c>false</c>.
        /// </value>
        public bool ConfigurationChanged
        {
            get { return m_ConfigurationChanged; }
        }

        /// <summary>Store the configuration, i.e. write a specific configuration file.
        /// </summary>
        public void StoreConfiguration()
        {
            GnuplotApplication.StoreConfiguration(editPath.Text);
            m_ConfigurationChanged = false;
        }

        /// <summary>Restores the configuration, i.e. reads the original configuration from some
        /// configuration file or take some default values and show these configuration.
        /// </summary>
        public void RestoreConfiguration()
        {
            editPath.Text = GnuplotApplication.GetGnuPlotPathFromConfigFile();
        }
        #endregion

        #region private methods

        /// <summary>Handles the Click event of the <c>buttonPath</c> control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void bPath_Click(object sender, EventArgs e)
        {
            if (m_OpenFileDialog.ShowDialog() == DialogResult.OK)
            {
                editPath.Text = m_OpenFileDialog.FileName;
                m_ConfigurationChanged = true;
            }
        }

        /// <summary>Handles the Click event of the <c>buttonReset</c> control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void bReset_Click(object sender, EventArgs e)
        {
            RestoreConfiguration();
        }
        #endregion
    }
}