﻿/*
   Copyright (C) 2011-2012 Markus Wendt
   All rights reserved.
 
   Redistribution and use in source and binary forms, including commercial applications, with or without modification, 
   are permitted provided that the following conditions are met: 
 
   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
      Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following 
      disclaimer in the documentation and/or other materials provided with the distribution. 

   3. If you use this software in a product, an acknowledgment (see the following) in the product documentation is required. 
      The end-user documentation included with the redistribution, if any, must include the following acknowledgment: 

      "Dodoni.net (http://www.dodoni-project.net/) Copyright (C) 2011-2012 Markus Wendt" 

      Alternately, this acknowledgment may appear in the software itself, if and wherever such third-party acknowledgments normally appear. 

   4. Neither the name 'Dodoni.net' nor the names of its contributors may be used to endorse or promote products 
      derived from this software without specific prior written permission. For written permission, please contact info<at>dodoni-project.net. 

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
   BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
   SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
   For more information, please see http://www.dodoni-project.net/.
 */
using System;
using System.Text;
using System.Collections.Generic;

using Dodoni.BasicComponents;
using Dodoni.XLBasicComponents;

namespace Dodoni.XLIntegration.Utilities
{
    /// <summary>Serves as interface for processing user (Excel) input for gnuplot, i.e. for the values (or labels) of the x- or y-axis etc.
    /// </summary>
    public interface IGnuplotDataProcessing : IIdentifierNameable
    {
        /// <summary>Gets a specific (gnuplot) <see cref="System.String"/> representation of a specified Excel cell, i.e. the value (or label) of the x-, y-coordinate etc.
        /// </summary>
        /// <param name="xlData">The <see cref="IExcelDataQuery"/> representation of the user input.</param>
        /// <param name="rowIndex">Thr null-based row index of the Excel cell to convert.</param>
        /// <param name="processedData">The (perhaps gnuplot specific) <see cref="System.String"/> representation of the user input, i.e. of the specified Excel cell.</param>
        /// <param name="columnIndex">The null-based index of the column of <paramref name="xlData"/> to take into account.</param>
        /// <returns>A value indicating whether <paramref name="processedData"/> contains valid data.</returns>
        ExcelCellValueState TryGetDataAsString(IExcelDataQuery xlData, int rowIndex, out string processedData, int columnIndex = 0);

        /// <summary>Gets a preliminary gnuplot command.
        /// </summary>
        /// <param name="gnuplotAxis">The gnuplot axis.</param>
        /// <returns>One or more gnuplot command needed for gnuplot to interpret the user input in a correct way; perhaps <c>null</c>.</returns>
        /// <remarks>For example
        /// <para>
        ///   set xdata time
        ///   set timefmt "%Y-%m-%d" 
        /// </para>
        /// for a specific <see cref="System.DateTime"/> input w.r.t. x axis.
        ///  </remarks>
        string GetPreliminaryGnuplotCommand(GnuplotAxis gnuplotAxis);

    }
}